from django.contrib import admin

# Helpers ##########


class ModelAdmin(admin.ModelAdmin):
    """ deletion of top level objects is evil """

    actions = None

    def has_delete_permission(self, request, obj=None):
        return False

    # we probably don't want to add anything through the interface
    def has_add_permission(self, request):
        return False

    # To ignore `DisallowedModelAdminLookup` error because of non
    # registered models
    def lookup_allowed(self, request, key):
        return True


class ReadOnlyTabularInline(admin.TabularInline):
    def has_add_permission(self, request):
        return False

    can_delete = False


class IdentifierInline(admin.TabularInline):
    fields = readonly_fields = ("identifier", "scheme")
    extra = 0
    can_delete = False
    verbose_name = "ID from another system"
    verbose_name_plural = "IDs from other systems"

    def has_add_permission(self, request):
        return False


class LinkInline(admin.TabularInline):
    fields = ("url", "note")
    extra = 0


class ContactDetailInline(admin.TabularInline):
    fields = ("type", "value", "note", "label")
    extra = 0
    verbose_name = "Piece of contact information"
    verbose_name_plural = "Contact information"


class OtherNameInline(admin.TabularInline):
    extra = 0
    verbose_name = "Alternate name"
    verbose_name_plural = "Alternate names"


class MimetypeLinkInline(admin.TabularInline):
    fields = ("media_type", "url")


class RelatedEntityInline(admin.TabularInline):
    fields = ("name", "entity_type", "organization", "person")



from ..utils import _make_pseudo_id
from .popolo import pseudo_organization
from .base import BaseModel, SourceMixin, AssociatedLinkMixin, cleanup_list
from .schemas.bill import schema


class Action(dict):
    def add_related_entity(self, name, entity_type, entity_id=None):
        ent = {"name": name, "entity_type": entity_type, entity_type + "_id": entity_id}
        self["related_entities"].append(ent)
        return ent


class Bill(SourceMixin, AssociatedLinkMixin, BaseModel):
    """
    An Open Civic Data bill.
    """

    _type = "bill"
    _schema = schema

    def __init__(
        self,
        identifier,
        legislative_session,
        title,
        *,
        chamber=None,
        from_organization=None,
        classification=None,
    ):
        super(Bill, self).__init__()

        self.identifier = identifier
        self.legislative_session = legislative_session
        self.title = title
        self.classification = cleanup_list(classification, ["bill"])
        self.from_organization = pseudo_organization(
            from_organization, chamber, "legislature"
        )

        self.actions = []
        self.other_identifiers = []
        self.other_titles = []
        self.documents = []
        self.related_bills = []
        self.sponsorships = []
        self.subject = []
        self.abstracts = []
        self.versions = []

    def pre_save(self, jurisdiction_id):
        # ensure subject is sorted for idempotent JSON output
        self.subject = sorted(self.subject)

    def add_action(
        self,
        description,
        date,
        *,
        organization=None,
        chamber=None,
        classification=None,
        related_entities=None,
    ):
        action = Action(
            description=description,
            date=date,
            organization_id=pseudo_organization(organization, chamber, "legislature"),
            classification=cleanup_list(classification, []),
            related_entities=[],
        )
        self.actions.append(action)
        return action

    def add_related_bill(self, identifier, legislative_session, relation_type):
        # will we need jurisdiction, organization?
        self.related_bills.append(
            {
                "identifier": identifier,
                "legislative_session": legislative_session,
                "relation_type": relation_type,
            }
        )

    def add_sponsorship(
        self,
        name,
        classification,
        entity_type,
        primary,
        *,
        chamber=None,
        entity_id=None,
    ):
        sp = {
            "name": name,
            "classification": classification,
            "entity_type": entity_type,
            "primary": primary,
            # set these so that all JSON objects have the same keys, prevents import errors
            "person_id": None,
            "organization_id": None,
        }
        # overwrite the id that exists
        if entity_type:
            if not entity_id:
                entity_id = _make_pseudo_id(name=name)
            sp[entity_type + "_id"] = entity_id
        self.sponsorships.append(sp)

    def add_sponsorship_by_identifier(
        self,
        name,
        classification,
        entity_type,
        primary,
        *,
        scheme,
        identifier,
        chamber=None,
    ):
        return self.add_sponsorship(
            name,
            classification,
            entity_type,
            primary,
            chamber=chamber,
            entity_id=_make_pseudo_id(
                identifiers__scheme=scheme, identifiers__identifier=identifier
            ),
        )

    def add_subject(self, subject):
        self.subject.append(subject)

    def add_abstract(self, abstract, note):
        self.abstracts.append({"note": note, "abstract": abstract})

    def add_title(self, title, note=""):
        self.other_titles.append({"note": note, "title": title})

    def add_identifier(self, identifier):
        self.other_identifiers.append({"identifier": identifier})

    def add_document_link(
        self,
        note,
        url,
        *,
        classification="",
        date="",
        media_type="",
        on_duplicate="warn",
    ):
        return self._add_associated_link(
            collection="documents",
            note=note,
            url=url,
            date=date,
            classification=classification,
            media_type=media_type,
            on_duplicate=on_duplicate,
        )

    def add_version_link(
        self,
        note,
        url,
        *,
        classification="",
        date="",
        media_type="",
        on_duplicate="warn",
    ):
        return self._add_associated_link(
            collection="versions",
            note=note,
            url=url,
            date=date,
            classification=classification,
            media_type=media_type,
            on_duplicate=on_duplicate,
        )

    def __str__(self):
        return self.identifier + " in " + self.legislative_session



# flake8: noqa
from .jurisdiction import Jurisdiction, JurisdictionScraper
from .popolo import Membership, Organization, Person, Post
from .vote_event import VoteEvent
from .bill import Bill
from .event import Event
from .base import Scraper, BaseBillScraper



# Generated by Django 3.0.5 on 2020-04-22 00:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("data", "0002_auto_20200422_0028")]

    operations = [
        migrations.RemoveField(model_name="membershiplink", name="membership"),
        migrations.DeleteModel(name="MembershipContactDetail"),
        migrations.DeleteModel(name="MembershipLink"),
    ]



class OpenStatesError(Exception):
    """ Base class for exceptions from core """


class InternalError(OpenStatesError):
    """ Indication something went wrong inside the backend that never should happen """


class CommandError(OpenStatesError):
    """ Errors from within CLI """


# import-related errors


class DataImportError(OpenStatesError):
    """ A generic error related to the import process. """


class InvalidVoteEventError(DataImportError):
    """ Attempt to create a vote event without an identifier or bill_id """


class NoMembershipsError(DataImportError):
    """ An attempt was made to import a person without any memberships. """

    def __init__(self, ids):
        super(NoMembershipsError, self).__init__(
            "no memberships for {} people: \n{}".format(len(ids), ", ".join(ids))
        )


class SameNameError(DataImportError):
    """ Attempt was made to import two people with the same name. """

    def __init__(self, name):
        super(SameNameError, self).__init__(
            'multiple people with same name "{}" in Jurisdiction '
            "- must provide birth_date to disambiguate".format(name)
        )


class SameOrgNameError(DataImportError):
    """ Attempt was made to import two orgs with the same name. """

    def __init__(self, name):
        super(SameOrgNameError, self).__init__(
            'multiple orgs with same name "{}" in Jurisdiction '.format(name)
        )


class DuplicateItemError(DataImportError):
    """ Attempt was made to import items that resolve to the same database item. """

    def __init__(self, data, obj, data_sources=None):
        super(DuplicateItemError, self).__init__(
            "attempt to import data that would conflict with "
            "data already in the import: {} "
            "(already imported as {})\n"
            "obj1 sources: {}\nobj2 sources: {}".format(
                data,
                obj,
                list(
                    obj.sources.values_list("url", flat=True)
                    if hasattr(obj, "sources")
                    else []
                ),
                [s["url"] for s in data_sources or []],
            )
        )


class UnresolvedIdError(DataImportError):
    """ Attempt was made to resolve an id that has no result. """


# scrape-related errors


class ScrapeError(OpenStatesError):
    """ A generic error related to the scrape process. """


class ScrapeValueError(OpenStatesError, ValueError):
    """ An invalid value was passed to a scrape object. """



import logging
from django.db import transaction
from django.db.models import Count, Subquery, OuterRef, Q, F
from .. import utils

# model imports are inside functions since this file is imported pre-init

logger = logging.getLogger("openstates")


def print_report(report):
    plan = report["plan"]
    print("{} ({})".format(plan["module"], ", ".join(plan["actions"])))
    for scraper, args in plan["scrapers"].items():
        print("  {}: {}".format(scraper, args))
    if "scrape" in report:
        for type, details in sorted(report["scrape"].items()):
            print(type + " scrape:")
            print("  duration: ", (details["end"] - details["start"]))
            print("  objects:")
            for objtype, num in sorted(details["objects"].items()):
                print("    {}: {}".format(objtype, num))
    if "import" in report:
        print("import:")
        for type, changes in sorted(report["import"].items()):
            if changes["insert"] or changes["update"] or changes["noop"]:
                print(
                    "  {}: {} new {} updated {} noop".format(
                        type, changes["insert"], changes["update"], changes["noop"]
                    )
                )


@transaction.atomic
def save_report(report, jurisdiction):
    from ..reports.models import RunPlan
    from ..data.models import Jurisdiction

    # set end time
    report["end"] = utils.utcnow()

    # if there's an error on the first run, the jurisdiction doesn't exist
    # yet, we opt for skipping creation of RunPlan until there's been at least
    # one good run
    try:
        Jurisdiction.objects.get(pk=jurisdiction)
    except Jurisdiction.DoesNotExist:
        logger.warning(
            "could not save RunPlan, no successful runs of {} yet".format(jurisdiction)
        )
        return

    plan = RunPlan.objects.create(
        jurisdiction_id=jurisdiction,
        success=report["success"],
        start_time=report["start"],
        end_time=report["end"],
        exception=report.get("exception", ""),
        traceback=report.get("traceback", ""),
    )

    for scraper, details in report.get("scrape", {}).items():
        args = " ".join(
            "{k}={v}".format(k=k, v=v)
            for k, v in report["plan"]["scrapers"].get(scraper, {}).items()
        )
        sr = plan.scrapers.create(
            scraper=scraper,
            args=args,
            start_time=details["start"],
            end_time=details["end"],
        )
        for object_type, num in details["objects"].items():
            sr.scraped_objects.create(object_type=object_type, count=num)

    for object_type, changes in report.get("import", {}).items():
        if changes["insert"] or changes["update"] or changes["noop"]:
            plan.imported_objects.create(
                object_type=object_type,
                insert_count=changes["insert"],
                update_count=changes["update"],
                noop_count=changes["noop"],
                start_time=changes["start"],
                end_time=changes["end"],
            )


def _simple_count(ModelCls, session, **filter):
    return (
        ModelCls.objects.filter(legislative_session_id=session).filter(**filter).count()
    )


def generate_session_report(session):
    from ..data.models import Bill, VoteEvent, VoteCount, PersonVote, BillSponsorship
    from ..reports.models import SessionDataQualityReport

    report = {
        "bills_missing_actions": _simple_count(Bill, session, actions__isnull=True),
        "bills_missing_sponsors": _simple_count(
            Bill, session, sponsorships__isnull=True
        ),
        "bills_missing_versions": _simple_count(Bill, session, versions__isnull=True),
        "votes_missing_bill": _simple_count(VoteEvent, session, bill__isnull=True),
        "votes_missing_voters": _simple_count(VoteEvent, session, votes__isnull=True),
        "votes_missing_yes_count": 0,
        "votes_missing_no_count": 0,
        "votes_with_bad_counts": 0,
    }

    voteevents = VoteEvent.objects.filter(legislative_session_id=session)
    # explicitly remove ordering pre-Django 3.1
    queryset = voteevents.order_by().annotate(
        yes_sum=Count("pk", filter=Q(votes__option="yes")),
        no_sum=Count("pk", filter=Q(votes__option="no")),
        other_sum=Count("pk", filter=Q(votes__option="other")),
        yes_count=Subquery(
            VoteCount.objects.filter(vote_event=OuterRef("pk"), option="yes").values(
                "value"
            )
        ),
        no_count=Subquery(
            VoteCount.objects.filter(vote_event=OuterRef("pk"), option="no").values(
                "value"
            )
        ),
        other_count=Subquery(
            VoteCount.objects.filter(vote_event=OuterRef("pk"), option="other").values(
                "value"
            )
        ),
    )

    for vote in queryset:
        if vote.yes_count is None:
            report["votes_missing_yes_count"] += 1
            vote.yes_count = 0
        if vote.no_count is None:
            report["votes_missing_no_count"] += 1
            vote.no_count = 0
        if vote.other_count is None:
            vote.other_count = 0
        if (
            vote.yes_sum != vote.yes_count
            or vote.no_sum != vote.no_count
            or vote.other_sum != vote.other_count
        ):
            report["votes_with_bad_counts"] += 1

    # handle unmatched
    queryset = (
        BillSponsorship.objects.filter(
            bill__legislative_session_id=session, entity_type="person", person_id=None
        )
        .values("name")
        .annotate(num=Count("name"))
    )
    report["unmatched_sponsor_people"] = {
        item["name"]: item["num"] for item in queryset
    }
    queryset = (
        BillSponsorship.objects.filter(
            bill__legislative_session_id=session,
            entity_type="organization",
            person_id=None,
        )
        .values("name")
        .annotate(num=Count("name"))
    )
    report["unmatched_sponsor_organizations"] = {
        item["name"]: item["num"] for item in queryset
    }
    queryset = (
        PersonVote.objects.filter(
            vote_event__legislative_session_id=session, voter__isnull=True
        )
        .values(name=F("voter_name"))
        .annotate(num=Count("voter_name"))
    )
    report["unmatched_voters"] = {item["name"]: item["num"] for item in queryset}

    # atomically replace the report if it exists
    new_report = SessionDataQualityReport(legislative_session_id=session, **report)
    with transaction.atomic():
        SessionDataQualityReport.objects.filter(legislative_session=session).delete()
        new_report.save()

    return new_report



from .ak import AK
from .al import AL
from .ar import AR
from .az import AZ
from .ca import CA
from .co import CO
from .ct import CT
from .dc import DC
from .de import DE
from .fl import FL
from .ga import GA
from .hi import HI
from .ia import IA
from .id import ID
from .il import IL
from .ind import IN
from .ks import KS
from .ky import KY
from .la import LA
from .ma import MA
from .md import MD
from .me import ME
from .mi import MI
from .mn import MN
from .mo import MO
from .ms import MS
from .mt import MT
from .nc import NC
from .nd import ND
from .ne import NE
from .nh import NH
from .nj import NJ
from .nm import NM
from .nv import NV
from .ny import NY
from .oh import OH
from .ok import OK
from .ore import OR
from .pa import PA
from .pr import PR
from .ri import RI
from .sc import SC
from .sd import SD
from .tn import TN
from .tx import TX
from .ut import UT
from .va import VA
from .vt import VT
from .wa import WA
from .wi import WI
from .wv import WV
from .wy import WY


STATES = [
    AK,
    AL,
    AR,
    AZ,
    CA,
    CO,
    CT,
    DC,
    DE,
    FL,
    GA,
    HI,
    IA,
    ID,
    IL,
    IN,
    KS,
    KY,
    LA,
    MA,
    MD,
    ME,
    MI,
    MN,
    MO,
    MS,
    MT,
    NC,
    ND,
    NE,
    NH,
    NJ,
    NM,
    NV,
    NY,
    OH,
    OK,
    OR,
    PA,
    PR,
    RI,
    SC,
    SD,
    TN,
    TX,
    UT,
    VA,
    VT,
    WA,
    WI,
    WV,
    WY,
]

STATES_BY_ABBR = {s.abbr: s for s in STATES}
STATES_BY_JID = {s.jurisdiction_id: s for s in STATES}
STATES_BY_NAME = {s.name.lower(): s for s in STATES}



# Generated by Django 3.0.4 on 2020-03-31 12:48

import django.contrib.postgres.fields
import django.contrib.postgres.fields.jsonb
import django.contrib.postgres.indexes
import django.contrib.postgres.search
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import openstates.data.models.base
import re
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Bill",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time of creation."
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="The date and time of the last update."
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        default=dict,
                        help_text="A key-value store for storing arbitrary information not covered elsewhere.",
                    ),
                ),
                (
                    "locked_fields",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "id",
                    openstates.data.models.base.OCDIDField(
                        ocd_type="bill",
                        serialize=False,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["UNICODE"],
                                message="ID must match ^ocd-bill/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                                regex="^ocd-bill/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                            )
                        ],
                    ),
                ),
                ("identifier", models.CharField(max_length=100)),
                ("title", models.TextField()),
                (
                    "classification",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "subject",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_bill"},
        ),
        migrations.CreateModel(
            name="BillAction",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("description", models.TextField()),
                ("date", models.CharField(max_length=25)),
                (
                    "classification",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                ("order", models.PositiveIntegerField()),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, default=dict
                    ),
                ),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="actions",
                        to="data.Bill",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billaction", "ordering": ["order"]},
        ),
        migrations.CreateModel(
            name="BillDocument",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("note", models.CharField(max_length=300)),
                ("date", models.CharField(max_length=10)),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, default=dict
                    ),
                ),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="documents",
                        to="data.Bill",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billdocument"},
        ),
        migrations.CreateModel(
            name="BillVersion",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("note", models.CharField(max_length=300)),
                ("date", models.CharField(max_length=10)),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, default=dict
                    ),
                ),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="versions",
                        to="data.Bill",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billversion"},
        ),
        migrations.CreateModel(
            name="BillVersionLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("media_type", models.CharField(max_length=100)),
                ("url", models.URLField(max_length=2000)),
                ("text", models.TextField(blank=True, default="")),
                (
                    "version",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.BillVersion",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billversionlink"},
        ),
        migrations.CreateModel(
            name="Division",
            fields=[
                (
                    "id",
                    models.CharField(max_length=300, primary_key=True, serialize=False),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="The name of the division.", max_length=300
                    ),
                ),
                (
                    "country",
                    models.CharField(
                        help_text="An ISO-3166-1 alpha-2 code identifying the county where this division is found.",
                        max_length=2,
                    ),
                ),
                (
                    "subtype1",
                    models.CharField(
                        blank=True,
                        help_text="The first subtype in the unique identifier.",
                        max_length=50,
                    ),
                ),
                (
                    "subid1",
                    models.CharField(
                        blank=True,
                        help_text="The first subidentifier in the unique identifer.",
                        max_length=100,
                    ),
                ),
                (
                    "subtype2",
                    models.CharField(
                        blank=True,
                        help_text="The second subtype in the unique identifier.",
                        max_length=50,
                    ),
                ),
                (
                    "subid2",
                    models.CharField(
                        blank=True,
                        help_text="The second subidentifier in the unique identifer.",
                        max_length=100,
                    ),
                ),
                (
                    "subtype3",
                    models.CharField(
                        blank=True,
                        help_text="The third subtype in the unique identifier.",
                        max_length=50,
                    ),
                ),
                (
                    "subid3",
                    models.CharField(
                        blank=True,
                        help_text="The third subidentifier in the unique identifer.",
                        max_length=100,
                    ),
                ),
                (
                    "subtype4",
                    models.CharField(
                        blank=True,
                        help_text="The fourth subtype in the unique identifier.",
                        max_length=50,
                    ),
                ),
                (
                    "subid4",
                    models.CharField(
                        blank=True,
                        help_text="The fourth subidentifier in the unique identifer.",
                        max_length=100,
                    ),
                ),
                (
                    "subtype5",
                    models.CharField(
                        blank=True,
                        help_text="The fifth subtype in the unique identifier.",
                        max_length=50,
                    ),
                ),
                (
                    "subid5",
                    models.CharField(
                        blank=True,
                        help_text="The fifth subidentifier in the unique identifer.",
                        max_length=100,
                    ),
                ),
                (
                    "subtype6",
                    models.CharField(
                        blank=True,
                        help_text="The sixth subtype in the unique identifier.",
                        max_length=50,
                    ),
                ),
                (
                    "subid6",
                    models.CharField(
                        blank=True,
                        help_text="The sixth subidentifier in the unique identifer.",
                        max_length=100,
                    ),
                ),
                (
                    "subtype7",
                    models.CharField(
                        blank=True,
                        help_text="The seventh subtype in the unique identifier.",
                        max_length=50,
                    ),
                ),
                (
                    "subid7",
                    models.CharField(
                        blank=True,
                        help_text="The seventh subidentifier in the unique identifer.",
                        max_length=100,
                    ),
                ),
                (
                    "redirect",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Division",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_division"},
        ),
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time of creation."
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="The date and time of the last update."
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        default=dict,
                        help_text="A key-value store for storing arbitrary information not covered elsewhere.",
                    ),
                ),
                (
                    "locked_fields",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "id",
                    openstates.data.models.base.OCDIDField(
                        ocd_type="event",
                        serialize=False,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["UNICODE"],
                                message="ID must match ^ocd-event/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                                regex="^ocd-event/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                            )
                        ],
                    ),
                ),
                ("name", models.CharField(max_length=1000)),
                ("description", models.TextField()),
                ("classification", models.CharField(max_length=100)),
                ("start_date", models.CharField(max_length=25)),
                ("end_date", models.CharField(blank=True, max_length=25)),
                ("all_day", models.BooleanField(default=False)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("cancelled", "Cancelled"),
                            ("tentative", "Tentative"),
                            ("confirmed", "Confirmed"),
                            ("passed", "Passed"),
                        ],
                        max_length=20,
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_event"},
        ),
        migrations.CreateModel(
            name="EventAgendaItem",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("description", models.TextField()),
                (
                    "classification",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                ("order", models.CharField(blank=True, max_length=100)),
                (
                    "subjects",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "notes",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, default=dict
                    ),
                ),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="agenda",
                        to="data.Event",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventagendaitem"},
        ),
        migrations.CreateModel(
            name="EventAgendaMedia",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("note", models.CharField(max_length=300)),
                ("date", models.CharField(blank=True, max_length=25)),
                ("offset", models.PositiveIntegerField(null=True)),
                (
                    "agenda_item",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="media",
                        to="data.EventAgendaItem",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventagendamedia"},
        ),
        migrations.CreateModel(
            name="EventDocument",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("note", models.CharField(max_length=300)),
                ("date", models.CharField(blank=True, max_length=25)),
                (
                    "classification",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("agenda", "Agenda"),
                            ("minutes", "Minutes"),
                            ("transcript", "Transcript"),
                            ("testimony", "Testimony"),
                        ],
                        max_length=50,
                    ),
                ),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="documents",
                        to="data.Event",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventdocument"},
        ),
        migrations.CreateModel(
            name="EventMedia",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("note", models.CharField(max_length=300)),
                ("date", models.CharField(blank=True, max_length=25)),
                ("offset", models.PositiveIntegerField(null=True)),
                (
                    "classification",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("audio recording", "Audio Recording"),
                            ("video recording", "Video Recording"),
                        ],
                        max_length=50,
                    ),
                ),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="media",
                        to="data.Event",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventmedia"},
        ),
        migrations.CreateModel(
            name="Jurisdiction",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time of creation."
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="The date and time of the last update."
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        default=dict,
                        help_text="A key-value store for storing arbitrary information not covered elsewhere.",
                    ),
                ),
                (
                    "locked_fields",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "id",
                    openstates.data.models.base.OCDIDField(
                        ocd_type="jurisdiction",
                        serialize=False,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["UNICODE"],
                                message="ID must match ^ocd-jurisdiction/country:[a-z]{2}(/[^\\W\\d]+:[\\w.~-]+)*/\\w+$",
                                regex="^ocd-jurisdiction/country:[a-z]{2}(/[^\\W\\d]+:[\\w.~-]+)*/\\w+$",
                            )
                        ],
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="The common name of the Jurisdiction, such as 'Wyoming.'",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="The primary website of the Jurisdiction.",
                        max_length=2000,
                    ),
                ),
                (
                    "classification",
                    models.CharField(
                        choices=[
                            ("government", "Government"),
                            ("legislature", "Legislature"),
                            ("executive", "Executive"),
                            ("school", "School System"),
                            ("park", "Park District"),
                            ("sewer", "Sewer District"),
                            ("forest", "Forest Preserve District"),
                            ("transit_authority", "Transit Authority"),
                        ],
                        db_index=True,
                        default="government",
                        help_text="The type of Jurisdiction being defined.",
                        max_length=50,
                    ),
                ),
                (
                    "feature_flags",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        help_text="A list of features that are present for data in this jurisdiction.",
                        size=None,
                    ),
                ),
                (
                    "division",
                    models.ForeignKey(
                        help_text="A link to a Division related to this Jurisdiction.",
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="jurisdictions",
                        to="data.Division",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_jurisdiction"},
        ),
        migrations.CreateModel(
            name="LegislativeSession",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("identifier", models.CharField(max_length=100)),
                ("name", models.CharField(max_length=300)),
                (
                    "classification",
                    models.CharField(
                        blank=True,
                        choices=[("primary", "Primary"), ("special", "Special")],
                        max_length=100,
                    ),
                ),
                ("start_date", models.CharField(max_length=10)),
                ("end_date", models.CharField(max_length=10)),
                (
                    "jurisdiction",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="legislative_sessions",
                        to="data.Jurisdiction",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_legislativesession"},
        ),
        migrations.CreateModel(
            name="Membership",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time of creation."
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="The date and time of the last update."
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        default=dict,
                        help_text="A key-value store for storing arbitrary information not covered elsewhere.",
                    ),
                ),
                (
                    "locked_fields",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "id",
                    openstates.data.models.base.OCDIDField(
                        ocd_type="membership",
                        serialize=False,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["UNICODE"],
                                message="ID must match ^ocd-membership/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                                regex="^ocd-membership/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                            )
                        ],
                    ),
                ),
                (
                    "person_name",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="The name of the Person that is a member of the Organization.",
                        max_length=300,
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        blank=True,
                        help_text="A label describing the membership.",
                        max_length=300,
                    ),
                ),
                (
                    "role",
                    models.CharField(
                        blank=True,
                        help_text="The role that the member fulfills in the Organization.",
                        max_length=300,
                    ),
                ),
                (
                    "start_date",
                    models.CharField(
                        blank=True,
                        help_text="The date on which the relationship began in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "end_date",
                    models.CharField(
                        blank=True,
                        help_text="The date on which the relationship ended in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_membership"},
        ),
        migrations.CreateModel(
            name="Organization",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time of creation."
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="The date and time of the last update."
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        default=dict,
                        help_text="A key-value store for storing arbitrary information not covered elsewhere.",
                    ),
                ),
                (
                    "locked_fields",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "id",
                    openstates.data.models.base.OCDIDField(
                        ocd_type="organization",
                        serialize=False,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["UNICODE"],
                                message="ID must match ^ocd-organization/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                                regex="^ocd-organization/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                            )
                        ],
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="The name of the Organization.", max_length=300
                    ),
                ),
                (
                    "image",
                    models.URLField(
                        blank=True,
                        help_text="A URL leading to an image that identifies the Organization visually.",
                        max_length=2000,
                    ),
                ),
                (
                    "classification",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("legislature", "Legislature"),
                            ("executive", "Executive"),
                            ("upper", "Upper Chamber"),
                            ("lower", "Lower Chamber"),
                            ("party", "Party"),
                            ("committee", "Committee"),
                            ("commission", "Commission"),
                            ("corporation", "Corporation"),
                            ("agency", "Agency"),
                            ("department", "Department"),
                            ("judiciary", "Judiciary"),
                        ],
                        help_text="The type of Organization being defined.",
                        max_length=100,
                    ),
                ),
                (
                    "founding_date",
                    models.CharField(
                        blank=True,
                        help_text="The founding date of the Organization in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "dissolution_date",
                    models.CharField(
                        blank=True,
                        help_text="The dissolution date of the Organization in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "jurisdiction",
                    models.ForeignKey(
                        help_text="A link to the Jurisdiction that contains this Organization.",
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="organizations",
                        to="data.Jurisdiction",
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        help_text="A link to another Organization that serves as this Organization's parent.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="children",
                        to="data.Organization",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_organization"},
        ),
        migrations.CreateModel(
            name="Person",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time of creation."
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="The date and time of the last update."
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        default=dict,
                        help_text="A key-value store for storing arbitrary information not covered elsewhere.",
                    ),
                ),
                (
                    "locked_fields",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "id",
                    openstates.data.models.base.OCDIDField(
                        ocd_type="person",
                        serialize=False,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["UNICODE"],
                                message="ID must match ^ocd-person/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                                regex="^ocd-person/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                            )
                        ],
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="A Person's preferred full name.",
                        max_length=300,
                    ),
                ),
                (
                    "sort_name",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="A version of a Person's full name rearranged for alphabetical sorting.",
                        max_length=100,
                    ),
                ),
                (
                    "family_name",
                    models.CharField(
                        blank=True, help_text="A Person's family name.", max_length=100
                    ),
                ),
                (
                    "given_name",
                    models.CharField(
                        blank=True, help_text="A Person's given name.", max_length=100
                    ),
                ),
                (
                    "image",
                    models.URLField(
                        blank=True,
                        help_text="A URL leading to an image that identifies the Person visually.",
                        max_length=2000,
                    ),
                ),
                (
                    "gender",
                    models.CharField(
                        blank=True, help_text="A Person's gender", max_length=100
                    ),
                ),
                (
                    "summary",
                    models.CharField(
                        blank=True,
                        help_text="A short, one-line account of a Person's life.",
                        max_length=500,
                    ),
                ),
                (
                    "national_identity",
                    models.CharField(
                        blank=True,
                        help_text="The nation a Person is identified with.",
                        max_length=300,
                    ),
                ),
                (
                    "biography",
                    models.TextField(
                        blank=True, help_text="An extended account of a Person's life."
                    ),
                ),
                (
                    "birth_date",
                    models.CharField(
                        blank=True,
                        help_text="The date of a Person's birth in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "death_date",
                    models.CharField(
                        blank=True,
                        help_text="The date of a Person's death in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "people",
                "db_table": "opencivicdata_person",
            },
        ),
        migrations.CreateModel(
            name="Post",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time of creation."
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="The date and time of the last update."
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        default=dict,
                        help_text="A key-value store for storing arbitrary information not covered elsewhere.",
                    ),
                ),
                (
                    "locked_fields",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "id",
                    openstates.data.models.base.OCDIDField(
                        ocd_type="post",
                        serialize=False,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["UNICODE"],
                                message="ID must match ^ocd-post/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                                regex="^ocd-post/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                            )
                        ],
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        help_text="A label describing the Post.", max_length=300
                    ),
                ),
                (
                    "role",
                    models.CharField(
                        blank=True,
                        help_text="The function that the holder of the post fulfills.",
                        max_length=300,
                    ),
                ),
                (
                    "start_date",
                    models.CharField(
                        blank=True,
                        help_text="An optional start date for the Post in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "end_date",
                    models.CharField(
                        blank=True,
                        help_text="An optional end date for the Post in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "maximum_memberships",
                    models.PositiveIntegerField(
                        default=1,
                        help_text="The maximum number of people who can hold this Post.",
                    ),
                ),
                (
                    "division",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        help_text="The Division where the post exists.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="posts",
                        to="data.Division",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        help_text="The Organization in which the post is held.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="posts",
                        to="data.Organization",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_post"},
        ),
        migrations.CreateModel(
            name="VoteEvent",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="The date and time of creation."
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="The date and time of the last update."
                    ),
                ),
                (
                    "locked_fields",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                (
                    "id",
                    openstates.data.models.base.OCDIDField(
                        ocd_type="vote",
                        serialize=False,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["UNICODE"],
                                message="ID must match ^ocd-vote/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                                regex="^ocd-vote/[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}$",
                            )
                        ],
                    ),
                ),
                ("identifier", models.CharField(blank=True, max_length=300)),
                ("motion_text", models.TextField()),
                (
                    "motion_classification",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(),
                        blank=True,
                        default=list,
                        size=None,
                    ),
                ),
                ("start_date", models.CharField(max_length=25)),
                ("end_date", models.CharField(blank=True, max_length=25)),
                (
                    "result",
                    models.CharField(
                        choices=[("pass", "Pass"), ("fail", "Fail")], max_length=50
                    ),
                ),
                (
                    "extras",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, default=dict
                    ),
                ),
                (
                    "bill",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="votes",
                        to="data.Bill",
                    ),
                ),
                (
                    "bill_action",
                    models.OneToOneField(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="vote",
                        to="data.BillAction",
                    ),
                ),
                (
                    "legislative_session",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="votes",
                        to="data.LegislativeSession",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="votes",
                        to="data.Organization",
                    ),
                ),
            ],
            options={
                "db_table": "opencivicdata_voteevent",
                "index_together": {
                    ("legislative_session", "bill"),
                    ("legislative_session", "identifier", "bill"),
                },
            },
        ),
        migrations.CreateModel(
            name="VoteSource",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "vote_event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sources",
                        to="data.VoteEvent",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_votesource"},
        ),
        migrations.CreateModel(
            name="VoteCount",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "option",
                    models.CharField(
                        choices=[
                            ("yes", "Yes"),
                            ("no", "No"),
                            ("absent", "Absent"),
                            ("abstain", "Abstain"),
                            ("not voting", "Not Voting"),
                            ("paired", "Paired"),
                            ("excused", "Excused"),
                            ("other", "Other"),
                        ],
                        max_length=50,
                    ),
                ),
                ("value", models.PositiveIntegerField()),
                (
                    "vote_event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="counts",
                        to="data.VoteEvent",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_votecount"},
        ),
        migrations.CreateModel(
            name="SearchableBill",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "search_vector",
                    django.contrib.postgres.search.SearchVectorField(default=None),
                ),
                ("all_titles", models.TextField(default="")),
                ("raw_text", models.TextField(default="")),
                ("is_error", models.BooleanField(default=False)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "bill",
                    models.OneToOneField(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="searchable",
                        to="data.Bill",
                    ),
                ),
                (
                    "version_link",
                    models.OneToOneField(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="searchable",
                        to="data.BillVersionLink",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_searchablebill"},
        ),
        migrations.CreateModel(
            name="RelatedBill",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("identifier", models.CharField(max_length=100)),
                ("legislative_session", models.CharField(max_length=100)),
                (
                    "relation_type",
                    models.CharField(
                        choices=[
                            ("companion", "Companion"),
                            ("prior-session", "Prior Session"),
                            ("replaced-by", "Replaced By"),
                            ("replaces", "Replaces"),
                        ],
                        max_length=100,
                    ),
                ),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="related_bills",
                        to="data.Bill",
                    ),
                ),
                (
                    "related_bill",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="related_bills_reverse",
                        to="data.Bill",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_relatedbill"},
        ),
        migrations.CreateModel(
            name="PostLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "post",
                    models.ForeignKey(
                        help_text="A reference to the Post connected to this link.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.Post",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_postlink"},
        ),
        migrations.CreateModel(
            name="PostContactDetail",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[
                            ("address", "Postal Address"),
                            ("email", "Email"),
                            ("url", "URL"),
                            ("fax", "Fax"),
                            ("text", "Text Phone"),
                            ("voice", "Voice Phone"),
                            ("video", "Video Phone"),
                            ("pager", "Pager"),
                            ("textphone", "Device for people with hearing impairment"),
                        ],
                        help_text="The type of Contact being defined.",
                        max_length=50,
                    ),
                ),
                (
                    "value",
                    models.CharField(
                        help_text="The content of the Contact information like a phone number or email address.",
                        max_length=300,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note about the Contact value.",
                        max_length=300,
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        blank=True,
                        help_text="A title for the content of the Contact.",
                        max_length=300,
                    ),
                ),
                (
                    "post",
                    models.ForeignKey(
                        help_text="A link to the Post connected to this contact.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="contact_details",
                        to="data.Post",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_postcontactdetail"},
        ),
        migrations.CreateModel(
            name="PersonVote",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "option",
                    models.CharField(
                        choices=[
                            ("yes", "Yes"),
                            ("no", "No"),
                            ("absent", "Absent"),
                            ("abstain", "Abstain"),
                            ("not voting", "Not Voting"),
                            ("paired", "Paired"),
                            ("excused", "Excused"),
                            ("other", "Other"),
                        ],
                        max_length=50,
                    ),
                ),
                ("voter_name", models.CharField(max_length=300)),
                ("note", models.TextField(blank=True)),
                (
                    "vote_event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="votes",
                        to="data.VoteEvent",
                    ),
                ),
                (
                    "voter",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="votes",
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_personvote"},
        ),
        migrations.CreateModel(
            name="PersonSource",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        help_text="A link to the Person connected to this source.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sources",
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_personsource"},
        ),
        migrations.CreateModel(
            name="PersonName",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True, help_text="An alternative name.", max_length=500
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note about alternative name.",
                        max_length=500,
                    ),
                ),
                (
                    "start_date",
                    models.CharField(
                        blank=True,
                        help_text="An optional start date for usage of the alternative name in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "end_date",
                    models.CharField(
                        blank=True,
                        help_text="An optional end date for usage of the alternative name in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        help_text="A link to the Person connected to this alternative name.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="other_names",
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_personname"},
        ),
        migrations.CreateModel(
            name="PersonLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        help_text="A reference to the Person connected to this link.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_personlink"},
        ),
        migrations.CreateModel(
            name="PersonIdentifier",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "identifier",
                    models.CharField(
                        help_text="A unique identifier developed by an upstream or third party source.",
                        max_length=300,
                    ),
                ),
                (
                    "scheme",
                    models.CharField(
                        help_text="The name of the service that created the identifier.",
                        max_length=300,
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        help_text="A link to the Person connected to this alternative identifier.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="identifiers",
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_personidentifier"},
        ),
        migrations.CreateModel(
            name="PersonContactDetail",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[
                            ("address", "Postal Address"),
                            ("email", "Email"),
                            ("url", "URL"),
                            ("fax", "Fax"),
                            ("text", "Text Phone"),
                            ("voice", "Voice Phone"),
                            ("video", "Video Phone"),
                            ("pager", "Pager"),
                            ("textphone", "Device for people with hearing impairment"),
                        ],
                        help_text="The type of Contact being defined.",
                        max_length=50,
                    ),
                ),
                (
                    "value",
                    models.CharField(
                        help_text="The content of the Contact information like a phone number or email address.",
                        max_length=300,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note about the Contact value.",
                        max_length=300,
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        blank=True,
                        help_text="A title for the content of the Contact.",
                        max_length=300,
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        help_text="A link to the Person connected to this contact.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="contact_details",
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_personcontactdetail"},
        ),
        migrations.CreateModel(
            name="OrganizationSource",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        help_text="A link to the Organization connected to this source.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sources",
                        to="data.Organization",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_organizationsource"},
        ),
        migrations.CreateModel(
            name="OrganizationName",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True, help_text="An alternative name.", max_length=500
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note about alternative name.",
                        max_length=500,
                    ),
                ),
                (
                    "start_date",
                    models.CharField(
                        blank=True,
                        help_text="An optional start date for usage of the alternative name in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "end_date",
                    models.CharField(
                        blank=True,
                        help_text="An optional end date for usage of the alternative name in YYYY[-MM[-DD]] string format.",
                        max_length=10,
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        help_text="A link to the Organization with this alternative name.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="other_names",
                        to="data.Organization",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_organizationname"},
        ),
        migrations.CreateModel(
            name="OrganizationLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        help_text="A reference to the Organization connected to this link.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.Organization",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_organizationlink"},
        ),
        migrations.CreateModel(
            name="OrganizationIdentifier",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "identifier",
                    models.CharField(
                        help_text="A unique identifier developed by an upstream or third party source.",
                        max_length=300,
                    ),
                ),
                (
                    "scheme",
                    models.CharField(
                        help_text="The name of the service that created the identifier.",
                        max_length=300,
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        help_text="Reference to the Organization identified by this alternative identifier.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="identifiers",
                        to="data.Organization",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_organizationidentifier"},
        ),
        migrations.CreateModel(
            name="OrganizationContactDetail",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[
                            ("address", "Postal Address"),
                            ("email", "Email"),
                            ("url", "URL"),
                            ("fax", "Fax"),
                            ("text", "Text Phone"),
                            ("voice", "Voice Phone"),
                            ("video", "Video Phone"),
                            ("pager", "Pager"),
                            ("textphone", "Device for people with hearing impairment"),
                        ],
                        help_text="The type of Contact being defined.",
                        max_length=50,
                    ),
                ),
                (
                    "value",
                    models.CharField(
                        help_text="The content of the Contact information like a phone number or email address.",
                        max_length=300,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note about the Contact value.",
                        max_length=300,
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        blank=True,
                        help_text="A title for the content of the Contact.",
                        max_length=300,
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        help_text="A link to the Organization connected to this contact.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="contact_details",
                        to="data.Organization",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_organizationcontactdetail"},
        ),
        migrations.CreateModel(
            name="MembershipLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "membership",
                    models.ForeignKey(
                        help_text="A reference to the Membership connected to this link.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.Membership",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_membershiplink"},
        ),
        migrations.CreateModel(
            name="MembershipContactDetail",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[
                            ("address", "Postal Address"),
                            ("email", "Email"),
                            ("url", "URL"),
                            ("fax", "Fax"),
                            ("text", "Text Phone"),
                            ("voice", "Voice Phone"),
                            ("video", "Video Phone"),
                            ("pager", "Pager"),
                            ("textphone", "Device for people with hearing impairment"),
                        ],
                        help_text="The type of Contact being defined.",
                        max_length=50,
                    ),
                ),
                (
                    "value",
                    models.CharField(
                        help_text="The content of the Contact information like a phone number or email address.",
                        max_length=300,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note about the Contact value.",
                        max_length=300,
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        blank=True,
                        help_text="A title for the content of the Contact.",
                        max_length=300,
                    ),
                ),
                (
                    "membership",
                    models.ForeignKey(
                        help_text="A link to the Membership connected to this contact.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="contact_details",
                        to="data.Membership",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_membershipcontactdetail"},
        ),
        migrations.AddField(
            model_name="membership",
            name="on_behalf_of",
            field=models.ForeignKey(
                help_text="The Organization on whose behalf the Person is a member of the Organization.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="memberships_on_behalf_of",
                to="data.Organization",
            ),
        ),
        migrations.AddField(
            model_name="membership",
            name="organization",
            field=models.ForeignKey(
                help_text="A link to the Organization in which the Person is a member.",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="memberships",
                to="data.Organization",
            ),
        ),
        migrations.AddField(
            model_name="membership",
            name="person",
            field=models.ForeignKey(
                help_text="A link to the Person that is a member of the Organization.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="memberships",
                to="data.Person",
            ),
        ),
        migrations.AddField(
            model_name="membership",
            name="post",
            field=models.ForeignKey(
                help_text="\tThe Post held by the member in the Organization.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="memberships",
                to="data.Post",
            ),
        ),
        migrations.CreateModel(
            name="EventSource",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sources",
                        to="data.Event",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventsource"},
        ),
        migrations.CreateModel(
            name="EventRelatedEntity",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(max_length=2000)),
                ("entity_type", models.CharField(blank=True, max_length=20)),
                ("note", models.TextField()),
                (
                    "agenda_item",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="related_entities",
                        to="data.EventAgendaItem",
                    ),
                ),
                (
                    "bill",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Bill",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Organization",
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Person",
                    ),
                ),
                (
                    "vote_event",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.VoteEvent",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventrelatedentity"},
        ),
        migrations.CreateModel(
            name="EventParticipant",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(max_length=2000)),
                ("entity_type", models.CharField(blank=True, max_length=20)),
                ("note", models.TextField()),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="participants",
                        to="data.Event",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Organization",
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventparticipant"},
        ),
        migrations.CreateModel(
            name="EventMediaLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("media_type", models.CharField(max_length=100)),
                ("url", models.URLField(max_length=2000)),
                ("text", models.TextField(blank=True, default="")),
                (
                    "media",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.EventMedia",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventmedialink"},
        ),
        migrations.CreateModel(
            name="EventLocation",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(max_length=200)),
                ("url", models.URLField(blank=True, max_length=2000)),
                (
                    "jurisdiction",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="event_locations",
                        to="data.Jurisdiction",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventlocation"},
        ),
        migrations.CreateModel(
            name="EventLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.Event",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventlink"},
        ),
        migrations.CreateModel(
            name="EventDocumentLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("media_type", models.CharField(max_length=100)),
                ("url", models.URLField(max_length=2000)),
                ("text", models.TextField(blank=True, default="")),
                (
                    "document",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.EventDocument",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventdocumentlink"},
        ),
        migrations.CreateModel(
            name="EventAgendaMediaLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("media_type", models.CharField(max_length=100)),
                ("url", models.URLField(max_length=2000)),
                ("text", models.TextField(blank=True, default="")),
                (
                    "media",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.EventAgendaMedia",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_eventagendamedialink"},
        ),
        migrations.AddField(
            model_name="event",
            name="jurisdiction",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="events",
                to="data.Jurisdiction",
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="location",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="data.EventLocation",
            ),
        ),
        migrations.CreateModel(
            name="BillTitle",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("title", models.TextField()),
                ("note", models.TextField(blank=True)),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="other_titles",
                        to="data.Bill",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billtitle"},
        ),
        migrations.CreateModel(
            name="BillSponsorship",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(max_length=2000)),
                ("entity_type", models.CharField(blank=True, max_length=20)),
                ("primary", models.BooleanField(default=False)),
                ("classification", models.CharField(max_length=100)),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sponsorships",
                        to="data.Bill",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Organization",
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billsponsorship"},
        ),
        migrations.CreateModel(
            name="BillSource",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "note",
                    models.CharField(
                        blank=True,
                        help_text="A short, optional note related to an object.",
                        max_length=300,
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text="A hyperlink related to an object.", max_length=2000
                    ),
                ),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sources",
                        to="data.Bill",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billsource"},
        ),
        migrations.CreateModel(
            name="BillIdentifier",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "identifier",
                    models.CharField(
                        help_text="A unique identifier developed by an upstream or third party source.",
                        max_length=300,
                    ),
                ),
                (
                    "scheme",
                    models.CharField(
                        help_text="The name of the service that created the identifier.",
                        max_length=300,
                    ),
                ),
                ("note", models.TextField(blank=True)),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="other_identifiers",
                        to="data.Bill",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billidentifier"},
        ),
        migrations.CreateModel(
            name="BillDocumentLink",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("media_type", models.CharField(max_length=100)),
                ("url", models.URLField(max_length=2000)),
                ("text", models.TextField(blank=True, default="")),
                (
                    "document",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="data.BillDocument",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billdocumentlink"},
        ),
        migrations.CreateModel(
            name="BillActionRelatedEntity",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(max_length=2000)),
                ("entity_type", models.CharField(blank=True, max_length=20)),
                (
                    "action",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="related_entities",
                        to="data.BillAction",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Organization",
                    ),
                ),
                (
                    "person",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="data.Person",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billactionrelatedentity"},
        ),
        migrations.AddField(
            model_name="billaction",
            name="organization",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="actions",
                to="data.Organization",
            ),
        ),
        migrations.CreateModel(
            name="BillAbstract",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("abstract", models.TextField()),
                ("note", models.TextField(blank=True)),
                ("date", models.TextField(blank=True, max_length=10)),
                (
                    "bill",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="abstracts",
                        to="data.Bill",
                    ),
                ),
            ],
            options={"db_table": "opencivicdata_billabstract"},
        ),
        migrations.AddField(
            model_name="bill",
            name="from_organization",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="bills",
                to="data.Organization",
            ),
        ),
        migrations.AddField(
            model_name="bill",
            name="legislative_session",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="bills",
                to="data.LegislativeSession",
            ),
        ),
        migrations.AddIndex(
            model_name="searchablebill",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["search_vector"], name="search_index"
            ),
        ),
        migrations.AlterIndexTogether(
            name="post", index_together={("organization", "label")}
        ),
        migrations.AlterIndexTogether(
            name="organization",
            index_together={
                ("classification", "name"),
                ("jurisdiction", "classification", "name"),
            },
        ),
        migrations.AlterIndexTogether(
            name="membership",
            index_together={("organization", "person", "label", "post")},
        ),
        migrations.AlterIndexTogether(
            name="event", index_together={("jurisdiction", "start_date", "name")}
        ),
        migrations.AlterIndexTogether(
            name="bill",
            index_together={("from_organization", "legislative_session", "identifier")},
        ),
    ]



from .data import STATES_BY_ABBR, STATES_BY_JID, STATES_BY_NAME


def lookup(*, abbr=None, jurisdiction_id=None, name=None):
    if abbr:
        return STATES_BY_ABBR[abbr.upper()]
    if name:
        return STATES_BY_NAME[name.lower()]
    if jurisdiction_id:
        return STATES_BY_JID[jurisdiction_id]


def lookup_district_with_ancestors(*, division_id):
    pieces = division_id.split("/")
    if len(pieces) == 4:
        state_portion = pieces[2]
    elif len(pieces) == 3:
        # special case the at-large ids for now, these should be fixed
        if division_id == "ocd-division/country:us/territory:pr":
            state = lookup(abbr="pr")
            chamber = state.upper
        elif division_id == "ocd-division/country:us/district:dc":
            state = lookup(abbr="dc")
            chamber = state.legislature
        district = chamber.lookup_district(name="At-Large")
        return state, chamber, district
    else:
        raise ValueError(f"invalid division id: {division_id}")

    try:
        state = lookup(abbr=state_portion.split(":")[1])
    except KeyError:
        raise ValueError(f"invalid division id: {division_id}, no such state")

    district = state.lookup_district(division_id)

    if not district:
        raise ValueError(f"invalid division id: {division_id}, no such district")

    if state.unicameral:
        chamber = state.legislature
    elif "sldl" in division_id:
        chamber = state.lower
    elif "sldu" in division_id:
        chamber = state.upper

    return state, chamber, district



from .base import BaseModel, Scraper
from .schemas.jurisdiction import schema


class Jurisdiction(BaseModel):
    """ Base class for a jurisdiction """

    _type = "jurisdiction"
    _schema = schema

    # schema objects
    classification = None
    name = None
    url = None
    legislative_sessions = []
    extras = {}

    # non-db properties
    scrapers = {}
    default_scrapers = None
    ignored_scraped_sessions = []

    def __init__(self):
        super(BaseModel, self).__init__()
        self._related = []
        self.extras = {}

    @property
    def jurisdiction_id(self):
        return "{}/{}".format(
            self.division_id.replace("ocd-division", "ocd-jurisdiction"),
            self.classification,
        )

    _id = jurisdiction_id

    def as_dict(self):
        return {
            "_id": self.jurisdiction_id,
            "id": self.jurisdiction_id,
            "name": self.name,
            "url": self.url,
            "division_id": self.division_id,
            "classification": self.classification,
            "legislative_sessions": self.legislative_sessions,
            "extras": self.extras,
        }

    def __str__(self):
        return self.name

    def get_organizations(self):
        raise NotImplementedError(
            "get_organizations is not implemented"
        )  # pragma: no cover


class JurisdictionScraper(Scraper):
    def scrape(self):
        # yield a single Jurisdiction object
        yield self.jurisdiction

        # yield all organizations
        for org in self.jurisdiction.get_organizations():
            yield org



"""
Module for declaration of common constants available throughout Open Civic Data code.
"""

DIVISION_ID_REGEX = r"^ocd-division/country:[a-z]{2}(/[^\W\d]+:[\w.~-]+)*$"
JURISDICTION_ID_REGEX = r"^ocd-jurisdiction/country:[a-z]{2}(/[^\W\d]+:[\w.~-]+)*/\w+$"

# helper for making options-only lists
_keys = lambda allopts: [opt[0] for opt in allopts]  # noqa

"""
Policy on addition of new types here:

Because these lists are strictly enforced in lots of code for the purposes of data quality
we have a fairly liberal policy on amendment.

If a type is needed and is not duplicative of another type, it will be accepted.

At the moment, because of this policy, no method exists to extend these lists, instead we will
strive for them to be comprehensive.

The only exception to this would be translations, which should simply exist as translations of
the display name (2nd attribute).
"""

# NOTE: this list explicitly does not include RFC 6350s 'cell' as that is redundant with
# voice and the distinction will only lead to confusion.  contact_detail.note can be
# used to indicate if something is a home, work, cell, etc.
CONTACT_TYPE_CHOICES = (
    ("address", "Postal Address"),
    ("email", "Email"),
    ("url", "URL"),
    ("fax", "Fax"),
    ("text", "Text Phone"),
    ("voice", "Voice Phone"),
    ("video", "Video Phone"),
    ("pager", "Pager"),
    ("textphone", "Device for people with hearing impairment"),
)
CONTACT_TYPES = _keys(CONTACT_TYPE_CHOICES)


JURISDICTION_CLASSIFICATION_CHOICES = (
    ("state", "State"),
    ("municipality", "Municipality"),
)
JURISDICTION_CLASSIFICATIONS = _keys(JURISDICTION_CLASSIFICATION_CHOICES)


SESSION_CLASSIFICATION_CHOICES = (("primary", "Primary"), ("special", "Special"))
SESSION_CLASSIFICATIONS = _keys(SESSION_CLASSIFICATION_CHOICES)


ORGANIZATION_CLASSIFICATION_CHOICES = (
    ("legislature", "Legislature"),
    ("executive", "Executive"),
    ("upper", "Upper Chamber"),
    ("lower", "Lower Chamber"),
    ("party", "Party"),
    ("committee", "Committee"),
    ("government", "Government"),
)
ORGANIZATION_CLASSIFICATIONS = _keys(ORGANIZATION_CLASSIFICATION_CHOICES)

BILL_CLASSIFICATION_CHOICES = (
    ("bill", "Bill"),
    ("resolution", "Resolution"),
    ("concurrent resolution", "Concurrent Resolution"),
    ("joint resolution", "Joint Resolution"),
    ("memorial", "Memorial"),
    ("commemoration", "Commemoration"),
    ("concurrent memorial", "Concurrent Memorial"),
    ("joint memorial", "Joint Memorial"),
    ("proposed bill", "Proposed Bill"),
    ("proclamation", "Proclamation"),
    ("nomination", "Nomination"),
    ("contract", "Contract"),
    ("claim", "Claim"),
    ("appointment", "Appointment"),
    ("constitutional amendment", "Constitutional Amendment"),
    ("petition", "Petition"),
    ("order", "Order"),
    ("concurrent order", "Concurrent Order"),
    ("appropriation", "Appropriation"),
    ("ordinance", "Ordinance"),
    ("motion", "Motion"),
    ("study request", "Study Request"),
    ("concurrent study request", "Concurrent Study Request"),
    ("bill of address", "Bill of Address"),
)
BILL_CLASSIFICATIONS = _keys(BILL_CLASSIFICATION_CHOICES)

BILL_RELATION_TYPE_CHOICES = (
    ("companion", "Companion"),  # a companion in another chamber
    ("prior-session", "Prior Session"),  # an introduction from a prior session
    ("replaced-by", "Replaced By"),  # a bill has been replaced by another
    ("replaces", "Replaces"),  # a bill that replaces another
)
BILL_RELATION_TYPES = _keys(BILL_RELATION_TYPE_CHOICES)

BILL_ACTION_CLASSIFICATION_CHOICES = (
    ("filing", "Filing"),
    ("introduction", "Introduced"),
    ("enrolled", "Enrolled"),
    ("reading-1", "First Reading"),
    ("reading-2", "Second Reading"),
    ("reading-3", "Third Reading"),
    ("passage", "Passage"),
    ("failure", "Passage Failure"),
    ("withdrawal", "Withdrawal"),
    ("substitution", "Substitution"),
    ("amendment-introduction", "Amendment Introduction"),
    ("amendment-passage", "Amendment Passage"),
    ("amendment-withdrawal", "Amendment Withdrawal"),
    ("amendment-failure", "Amendment Failure"),
    ("amendment-amendment", "Amendment Amended"),
    ("amendment-deferral", "Amendment Deferred or Tabled"),
    ("committee-passage", "Passage from Committee"),
    ("committee-passage-favorable", "Favorable Passage from Committee"),
    ("committee-passage-unfavorable", "Unfavorable Passage from Committee"),
    ("committee-failure", "Failure in Committee"),
    ("executive-receipt", "Received By Executive"),
    ("executive-signature", "Signed By Executive"),
    ("executive-veto", "Veto By Executive"),
    ("executive-veto-line-item", "Line Item Veto By Executive"),
    ("became-law", "Became Law"),
    ("veto-override-passage", "Veto Override Passage"),
    ("veto-override-failure", "Veto Override Failure"),
    ("deferral", "Deferred or Tabled"),
    ("receipt", "Received"),
    ("referral", "Referred"),
    ("referral-committee", "Referred to Committee"),
)
BILL_ACTION_CLASSIFICATIONS = _keys(BILL_ACTION_CLASSIFICATION_CHOICES)

VOTE_CLASSIFICATION_CHOICES = (
    ("passage", "Passage in Chamber"),
    ("amendment", "Amendment Passage"),
    ("committee-passage", "Passage in Committee"),
    ("amendment", "Amendment"),
    ("reading-1", "First Reading"),
    ("reading-3", "Third Reading"),
)
VOTE_CLASSIFICATIONS = _keys(VOTE_CLASSIFICATION_CHOICES)

VOTE_OPTION_CHOICES = (
    ("yes", "Yes"),
    ("no", "No"),
    ("absent", "Absent"),
    ("abstain", "Abstain"),
    ("not voting", "Not Voting"),
    ("paired", "Paired"),
    ("excused", "Excused"),
    # Only for open states.
    ("other", "Other"),
)
VOTE_OPTIONS = _keys(VOTE_OPTION_CHOICES)

VOTE_RESULT_CHOICES = (("pass", "Pass"), ("fail", "Fail"))
VOTE_RESULTS = _keys(VOTE_RESULT_CHOICES)

BILL_VERSION_CHOICES = (
    ("", "Unknown"),
    ("filed", "Filed"),
    ("introduced", "Introduced"),
    ("amendment", "Amendment"),
    ("substituted", "Substituted"),
    ("enrolled", "Enrolled"),
    ("became-law", "Became Law"),
)
BILL_VERSION_CLASSIFICATIONS = _keys(BILL_VERSION_CHOICES)

BILL_DOCUMENT_CHOICES = (
    ("", "Unknown"),
    ("fiscal-note", "Fiscal Note"),
    ("committee-report", "Committee Report"),
    ("summary", "Summary"),
    ("digest", "Digest"),
    ("veto-message", "Veto Message"),
    ("analysis", "Analysis"),
    ("law", "Law"),
)
BILL_DOCUMENT_CLASSIFICATIONS = _keys(BILL_DOCUMENT_CHOICES)

EVENT_MEDIA_CLASSIFICATION_CHOICES = (
    ("audio recording", "Audio Recording"),
    ("video recording", "Video Recording"),
)
EVENT_MEDIA_CLASSIFICATIONS = _keys(EVENT_MEDIA_CLASSIFICATION_CHOICES)

EVENT_DOCUMENT_CLASSIFICATION_CHOICES = (
    ("agenda", "Agenda"),
    ("minutes", "Minutes"),
    ("transcript", "Transcript"),
    ("testimony", "Testimony"),
)
EVENT_DOCUMENT_CLASSIFICATIONS = _keys(EVENT_DOCUMENT_CLASSIFICATION_CHOICES)



from ..models import State, Chamber, simple_numbered_districts

CT = State(
    name="Connecticut",
    abbr="CT",
    capital="Hartford",
    capital_tz="America/New_York",
    fips="09",
    unicameral=False,
    legislature_name="Connecticut General Assembly",
    legislature_organization_id="ocd-organization/474ab9de-3ab4-4294-88c1-43ffd506870a",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/76921ea9-d120-566d-9612-ac1b715d0e68",
    division_id="ocd-division/country:us/state:ct",
    jurisdiction_id="ocd-jurisdiction/country:us/state:ct/government",
    url="http://www.cga.ct.gov/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/8e8c6d29-dcc1-4d2f-9ca5-82a4a3d3b962",
        num_seats=151,
        title="Representative",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:ct", "lower", 151
        ),
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/46a37f5a-a01b-4905-b023-a443b5be41c6",
        num_seats=36,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:ct", "upper", 36
        ),
    ),
)



# flake8: NOQA
from .jurisdiction import Jurisdiction, LegislativeSession
from .division import Division
from .people_orgs import (
    Organization,
    OrganizationLink,
    OrganizationSource,
    Person,
    PersonIdentifier,
    PersonName,
    PersonContactDetail,
    PersonLink,
    PersonSource,
    Post,
    Membership,
)
from .bill import (
    Bill,
    BillAbstract,
    BillTitle,
    BillIdentifier,
    RelatedBill,
    BillSponsorship,
    BillDocument,
    BillVersion,
    BillDocumentLink,
    BillVersionLink,
    BillSource,
    BillActionRelatedEntity,
    BillAction,
    SearchableBill,
)
from .vote import VoteEvent, VoteCount, PersonVote, VoteSource
from .event import (
    Event,
    EventLocation,
    EventMedia,
    EventMediaLink,
    EventDocument,
    EventLink,
    EventSource,
    EventParticipant,
    EventAgendaItem,
    EventRelatedEntity,
    EventAgendaMedia,
    EventAgendaMediaLink,
    EventDocumentLink,
)



# Generated by Django 3.0.5 on 2020-08-21 10:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("data", "0016_auto_20200811_1016"),
    ]

    operations = [
        migrations.RemoveField(model_name="person", name="summary",),
    ]



from ..models import State, Chamber, simple_numbered_districts

IL = State(
    name="Illinois",
    abbr="IL",
    capital="Springfield",
    capital_tz="America/Chicago",
    fips="17",
    unicameral=False,
    legislature_name="Illinois General Assembly",
    legislature_organization_id="ocd-organization/52332da7-9146-412e-bf64-a291e2757b63",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/1cc0938a-ec95-595c-ada8-3c9379af4704",
    division_id="ocd-division/country:us/state:il",
    jurisdiction_id="ocd-jurisdiction/country:us/state:il/government",
    url="http://www.ilga.gov/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/76120f9a-2982-4cc5-a6e2-de7401fa0d38",
        num_seats=118,
        title="Representative",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:il", "lower", 118
        ),
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/20782ae5-cc22-4470-846b-fd9788ad3db7",
        num_seats=59,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:il", "upper", 59
        ),
    ),
)



from ..models import State, Chamber, District

WV = State(
    name="West Virginia",
    abbr="WV",
    capital="Charleston",
    capital_tz="America/New_York",
    fips="54",
    unicameral=False,
    legislature_name="West Virginia Legislature",
    legislature_organization_id="ocd-organization/237dc1a5-2314-433e-887e-d63a970cf67d",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/193c7d7d-3161-5a34-a7b1-df72b553b7f3",
    division_id="ocd-division/country:us/state:wv",
    jurisdiction_id="ocd-jurisdiction/country:us/state:wv/government",
    url="http://www.legis.state.wv.us/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/3fcd7789-fa25-4d8b-a4b9-7c0d1d0f8bc4",
        num_seats=100,
        title="Delegate",
        districts=[
            District("1", "lower", "ocd-division/country:us/state:wv/sldl:1", 2),
            District("2", "lower", "ocd-division/country:us/state:wv/sldl:2", 1),
            District("3", "lower", "ocd-division/country:us/state:wv/sldl:3", 2),
            District("4", "lower", "ocd-division/country:us/state:wv/sldl:4", 2),
            District("5", "lower", "ocd-division/country:us/state:wv/sldl:5", 1),
            District("6", "lower", "ocd-division/country:us/state:wv/sldl:6", 1),
            District("7", "lower", "ocd-division/country:us/state:wv/sldl:7", 1),
            District("8", "lower", "ocd-division/country:us/state:wv/sldl:8", 1),
            District("9", "lower", "ocd-division/country:us/state:wv/sldl:9", 1),
            District("10", "lower", "ocd-division/country:us/state:wv/sldl:10", 3),
            District("11", "lower", "ocd-division/country:us/state:wv/sldl:11", 1),
            District("12", "lower", "ocd-division/country:us/state:wv/sldl:12", 1),
            District("13", "lower", "ocd-division/country:us/state:wv/sldl:13", 2),
            District("14", "lower", "ocd-division/country:us/state:wv/sldl:14", 1),
            District("15", "lower", "ocd-division/country:us/state:wv/sldl:15", 1),
            District("16", "lower", "ocd-division/country:us/state:wv/sldl:16", 3),
            District("17", "lower", "ocd-division/country:us/state:wv/sldl:17", 2),
            District("18", "lower", "ocd-division/country:us/state:wv/sldl:18", 1),
            District("19", "lower", "ocd-division/country:us/state:wv/sldl:19", 2),
            District("20", "lower", "ocd-division/country:us/state:wv/sldl:20", 1),
            District("21", "lower", "ocd-division/country:us/state:wv/sldl:21", 1),
            District("22", "lower", "ocd-division/country:us/state:wv/sldl:22", 2),
            District("23", "lower", "ocd-division/country:us/state:wv/sldl:23", 1),
            District("24", "lower", "ocd-division/country:us/state:wv/sldl:24", 2),
            District("25", "lower", "ocd-division/country:us/state:wv/sldl:25", 1),
            District("26", "lower", "ocd-division/country:us/state:wv/sldl:26", 1),
            District("27", "lower", "ocd-division/country:us/state:wv/sldl:27", 3),
            District("28", "lower", "ocd-division/country:us/state:wv/sldl:28", 2),
            District("29", "lower", "ocd-division/country:us/state:wv/sldl:29", 1),
            District("30", "lower", "ocd-division/country:us/state:wv/sldl:30", 1),
            District("31", "lower", "ocd-division/country:us/state:wv/sldl:31", 1),
            District("32", "lower", "ocd-division/country:us/state:wv/sldl:32", 3),
            District("33", "lower", "ocd-division/country:us/state:wv/sldl:33", 1),
            District("34", "lower", "ocd-division/country:us/state:wv/sldl:34", 1),
            District("35", "lower", "ocd-division/country:us/state:wv/sldl:35", 4),
            District("36", "lower", "ocd-division/country:us/state:wv/sldl:36", 3),
            District("37", "lower", "ocd-division/country:us/state:wv/sldl:37", 1),
            District("38", "lower", "ocd-division/country:us/state:wv/sldl:38", 1),
            District("39", "lower", "ocd-division/country:us/state:wv/sldl:39", 1),
            District("40", "lower", "ocd-division/country:us/state:wv/sldl:40", 1),
            District("41", "lower", "ocd-division/country:us/state:wv/sldl:41", 1),
            District("42", "lower", "ocd-division/country:us/state:wv/sldl:42", 2),
            District("43", "lower", "ocd-division/country:us/state:wv/sldl:43", 2),
            District("44", "lower", "ocd-division/country:us/state:wv/sldl:44", 1),
            District("45", "lower", "ocd-division/country:us/state:wv/sldl:45", 1),
            District("46", "lower", "ocd-division/country:us/state:wv/sldl:46", 1),
            District("47", "lower", "ocd-division/country:us/state:wv/sldl:47", 1),
            District("48", "lower", "ocd-division/country:us/state:wv/sldl:48", 4),
            District("49", "lower", "ocd-division/country:us/state:wv/sldl:49", 1),
            District("50", "lower", "ocd-division/country:us/state:wv/sldl:50", 3),
            District("51", "lower", "ocd-division/country:us/state:wv/sldl:51", 5),
            District("52", "lower", "ocd-division/country:us/state:wv/sldl:52", 1),
            District("53", "lower", "ocd-division/country:us/state:wv/sldl:53", 1),
            District("54", "lower", "ocd-division/country:us/state:wv/sldl:54", 1),
            District("55", "lower", "ocd-division/country:us/state:wv/sldl:55", 1),
            District("56", "lower", "ocd-division/country:us/state:wv/sldl:56", 1),
            District("57", "lower", "ocd-division/country:us/state:wv/sldl:57", 1),
            District("58", "lower", "ocd-division/country:us/state:wv/sldl:58", 1),
            District("59", "lower", "ocd-division/country:us/state:wv/sldl:59", 1),
            District("60", "lower", "ocd-division/country:us/state:wv/sldl:60", 1),
            District("61", "lower", "ocd-division/country:us/state:wv/sldl:61", 1),
            District("62", "lower", "ocd-division/country:us/state:wv/sldl:62", 1),
            District("63", "lower", "ocd-division/country:us/state:wv/sldl:63", 1),
            District("64", "lower", "ocd-division/country:us/state:wv/sldl:64", 1),
            District("65", "lower", "ocd-division/country:us/state:wv/sldl:65", 1),
            District("66", "lower", "ocd-division/country:us/state:wv/sldl:66", 1),
            District("67", "lower", "ocd-division/country:us/state:wv/sldl:67", 1),
        ],
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/660bc141-f4cb-4d98-b334-b12b637776ba",
        num_seats=34,
        title="Senator",
        districts=[
            District("1", "upper", "ocd-division/country:us/state:wv/sldu:1", 2),
            District("2", "upper", "ocd-division/country:us/state:wv/sldu:2", 2),
            District("3", "upper", "ocd-division/country:us/state:wv/sldu:3", 2),
            District("4", "upper", "ocd-division/country:us/state:wv/sldu:4", 2),
            District("5", "upper", "ocd-division/country:us/state:wv/sldu:5", 2),
            District("6", "upper", "ocd-division/country:us/state:wv/sldu:6", 2),
            District("7", "upper", "ocd-division/country:us/state:wv/sldu:7", 2),
            District("8", "upper", "ocd-division/country:us/state:wv/sldu:8", 2),
            District("9", "upper", "ocd-division/country:us/state:wv/sldu:9", 2),
            District("10", "upper", "ocd-division/country:us/state:wv/sldu:10", 2),
            District("11", "upper", "ocd-division/country:us/state:wv/sldu:11", 2),
            District("12", "upper", "ocd-division/country:us/state:wv/sldu:12", 2),
            District("13", "upper", "ocd-division/country:us/state:wv/sldu:13", 2),
            District("14", "upper", "ocd-division/country:us/state:wv/sldu:14", 2),
            District("15", "upper", "ocd-division/country:us/state:wv/sldu:15", 2),
            District("16", "upper", "ocd-division/country:us/state:wv/sldu:16", 2),
            District("17", "upper", "ocd-division/country:us/state:wv/sldu:17", 2),
        ],
    ),
)



# Generated by Django 3.0.5 on 2020-09-14 10:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("data", "0017_remove_person_summary"),
    ]

    operations = [
        migrations.RemoveField(model_name="bill", name="locked_fields",),
        migrations.RemoveField(model_name="event", name="locked_fields",),
        migrations.RemoveField(model_name="jurisdiction", name="locked_fields",),
        migrations.RemoveField(model_name="membership", name="locked_fields",),
        migrations.RemoveField(model_name="organization", name="locked_fields",),
        migrations.RemoveField(model_name="person", name="locked_fields",),
        migrations.RemoveField(model_name="post", name="locked_fields",),
        migrations.RemoveField(model_name="voteevent", name="locked_fields",),
    ]



from django.contrib import admin
from . import models


class ScrapeReportInline(admin.TabularInline):
    model = models.ScrapeReport
    readonly_fields = ("scraper", "args", "start_time", "end_time", "get_object_list")

    def has_add_permission(self, request):
        return False

    can_delete = False

    def get_object_list(self, obj):
        return "\n".join(
            "{} ({})".format(o.object_type, o.count) for o in obj.scraped_objects.all()
        )


class ImportObjectsInline(admin.TabularInline):
    model = models.ImportObjects
    readonly_fields = (
        "object_type",
        "insert_count",
        "update_count",
        "noop_count",
        "start_time",
        "end_time",
    )

    def has_add_permission(self, request):
        return False

    can_delete = False


@admin.register(models.RunPlan)
class RunPlanAdmin(admin.ModelAdmin):
    actions = None

    readonly_fields = (
        "jurisdiction",
        "success",
        "start_time",
        "end_time",
        "exception",
        "traceback",
    )
    list_filter = ("jurisdiction__name", "success")
    list_display = ("jurisdiction", "success", "start_time")
    inlines = [ScrapeReportInline, ImportObjectsInline]

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False


@admin.register(models.SessionDataQualityReport)
class SessionDataQualityAdmin(admin.ModelAdmin):
    actions = None

    readonly_fields = (
        "legislative_session",
        "bills_missing_actions",
        "bills_missing_sponsors",
        "bills_missing_versions",
        "votes_missing_voters",
        "votes_missing_bill",
        "votes_missing_yes_count",
        "votes_missing_no_count",
        "votes_with_bad_counts",
        "unmatched_sponsor_people",
        "unmatched_sponsor_organizations",
        "unmatched_voters",
    )
    list_display = (
        "jurisdiction_name",
        "legislative_session",
        "bills_missing_actions",
        "bills_missing_sponsors",
        "bills_missing_versions",
        "votes_missing_voters",
        "votes_missing_bill",
        "votes_missing_yes_count",
        "votes_missing_no_count",
        "votes_with_bad_counts",
    )
    list_filter = ("legislative_session__jurisdiction__name",)

    def jurisdiction_name(self, obj):
        return obj.legislative_session.jurisdiction.name



from ...data import common

contact_details = {
    "type": "array",
    "items": {
        "type": "object",
        "properties": {
            "type": {"type": "string", "enum": common.CONTACT_TYPES},
            "value": {"type": "string", "minLength": 1},
            "note": {"type": "string"},
            "label": {"type": "string"},
        },
    },
}

identifiers = {
    "items": {
        "properties": {
            "identifier": {"type": "string", "minLength": 1},
            "scheme": {"type": "string"},
        }
    },
    "type": "array",
}

fuzzy_date_string = {"type": "string", "pattern": "^[0-9]{4}(-[0-9]{2}){0,2}$"}
fuzzy_date_string_blank = {"type": "string", "pattern": "^([0-9]{4})?(-[0-9]{2}){0,2}$"}
fuzzy_datetime_string_blank = {
    "type": "string",
    "pattern": (
        "^([0-9]{4}((-[0-9]{2}){0,2}|(-[0-9]{2}){2}T"
        "[0-9]{2}(:[0-9]{2}){0,2}"
        "(Z|[+-][0-9]{2}(:[0-9]{2})?))?)?$"
    ),
}
fuzzy_date = {"type": [fuzzy_date_string, "date"]}
fuzzy_date_blank = {"type": [fuzzy_date_string_blank, "date"]}
fuzzy_datetime = {"type": [fuzzy_datetime_string_blank, "datetime"]}
fuzzy_datetime_blank = {"type": [fuzzy_datetime_string_blank, "datetime"]}

other_names = {
    "items": {
        "properties": {
            "name": {"type": "string", "minLength": 1},
            "start_date": fuzzy_date_blank,
            "end_date": fuzzy_date_blank,
            "note": {"type": "string"},
        },
        "type": "object",
    },
    "type": "array",
}


links = {
    "items": {
        "properties": {
            "note": {"type": "string"},
            "url": {"format": "uri", "type": "string"},
        },
        "type": "object",
    },
    "type": "array",
}


sources = {
    "items": {
        "properties": {
            "url": {"type": "string", "format": "uri"},
            "note": {"type": "string"},
        },
        "type": "object",
    },
    "minItems": 1,
    "type": "array",
}

extras = {"type": "object"}



# Generated by Django 3.0.5 on 2020-09-14 12:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("data", "0024_auto_20200914_1101"),
    ]

    operations = [
        migrations.RemoveField(model_name="billdocumentlink", name="text",),
        migrations.RemoveField(model_name="billversionlink", name="text",),
        migrations.RemoveField(model_name="eventagendamedialink", name="text",),
        migrations.RemoveField(model_name="eventdocumentlink", name="text",),
        migrations.RemoveField(model_name="eventmedialink", name="text",),
    ]



from ..models import State, Chamber, simple_numbered_districts

OK = State(
    name="Oklahoma",
    abbr="OK",
    capital="Oklahoma City",
    capital_tz="America/Chicago",
    fips="40",
    unicameral=False,
    legislature_name="Oklahoma Legislature",
    legislature_organization_id="ocd-organization/a720b7e9-ab67-405a-a209-473e9d6a6036",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/804e97df-c92a-577e-b610-545671bf7ed1",
    division_id="ocd-division/country:us/state:ok",
    jurisdiction_id="ocd-jurisdiction/country:us/state:ok/government",
    url="http://www.oklegislature.gov/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/cba1f18c-65a2-4eb0-886d-eada16da3d8a",
        num_seats=101,
        title="Representative",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:ok", "lower", 101
        ),
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/5c78775c-6ff3-4cf8-a5f2-4e5e550861f3",
        num_seats=48,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:ok", "upper", 48
        ),
    ),
)



# Generated by Django 3.0.5 on 2020-09-14 10:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("data", "0019_remove_billabstract_date"),
    ]

    operations = [
        migrations.RemoveField(model_name="billidentifier", name="note",),
        migrations.RemoveField(model_name="billidentifier", name="scheme",),
    ]



from django.contrib import admin
from django.template import defaultfilters
from .base import ModelAdmin, ReadOnlyTabularInline, IdentifierInline
from .. import models


class BillAbstractInline(ReadOnlyTabularInline):
    model = models.BillAbstract
    readonly_fields = ("abstract", "note")
    can_delete = False


class BillTitleInline(ReadOnlyTabularInline):
    model = models.BillTitle
    readonly_fields = ("title", "note")
    can_delete = False


class BillIdentifierInline(IdentifierInline):
    model = models.BillIdentifier
    fields = readonly_fields = ["identifier"]


class BillActionInline(ReadOnlyTabularInline):
    model = models.BillAction

    def get_related_entities(self, obj):
        ents = obj.related_entities.all()
        ent_list = [e.name for e in ents]
        return ", ".join(ent_list)

    get_related_entities.short_description = "Related Entities"
    get_related_entities.allow_tags = True

    list_select_related = ("BillActionRelatedEntity",)
    readonly_fields = ("date", "organization", "description", "get_related_entities")


class RelatedBillInline(ReadOnlyTabularInline):
    model = models.RelatedBill
    fk_name = "bill"
    readonly_fields = fields = ("identifier", "legislative_session", "relation_type")
    extra = 0


class BillSponsorshipInline(ReadOnlyTabularInline):
    model = models.BillSponsorship
    readonly_fields = fields = ("name", "primary", "classification")
    extra = 0


class DocVersionInline(ReadOnlyTabularInline):
    model = models.BillVersion

    def get_links(self, obj):
        return "<br />".join(
            '<a href="{0}">{0}</a>'.format(link.url) for link in obj.links.all()
        )

    get_links.short_description = "Links"
    get_links.allow_tags = True

    list_select_related = ("BillVersionLink",)
    readonly_fields = ("note", "date", "get_links")


class BillVersionInline(DocVersionInline):
    model = models.BillVersion


class BillDocumentInline(DocVersionInline):
    model = models.BillDocument


class BillSourceInline(ReadOnlyTabularInline):
    readonly_fields = ("url", "note")
    model = models.BillSource


@admin.register(models.Bill)
class BillAdmin(ModelAdmin):
    readonly_fields = fields = (
        "identifier",
        "legislative_session",
        "bill_classifications",
        "from_organization",
        "title",
        "id",
        "subject",
        "extras",
    )
    search_fields = ["identifier", "title"]
    list_select_related = ("legislative_session", "legislative_session__jurisdiction")
    inlines = [
        BillAbstractInline,
        BillTitleInline,
        BillIdentifierInline,
        BillActionInline,
        BillSponsorshipInline,
        BillSourceInline,
        RelatedBillInline,
        BillVersionInline,
        BillDocumentInline,
    ]

    def bill_classifications(self, obj):
        return ", ".join(obj.classification)

    def get_jurisdiction_name(self, obj):
        return obj.legislative_session.jurisdiction.name

    get_jurisdiction_name.short_description = "Jurisdiction"

    def get_session_name(self, obj):
        return obj.legislative_session.name

    get_session_name.short_description = "Session"
    get_session_name.admin_order_field = "legislative_session__name"

    def get_truncated_sponsors(self, obj):
        spons = ", ".join(s.name for s in obj.sponsorships.all()[:5])
        return defaultfilters.truncatewords(spons, 10)

    get_truncated_sponsors.short_description = "Sponsors"

    def get_truncated_title(self, obj):
        return defaultfilters.truncatewords(obj.title, 25)

    get_truncated_title.short_description = "Title"

    list_display = (
        "identifier",
        "get_jurisdiction_name",
        "get_session_name",
        "get_truncated_sponsors",
        "get_truncated_title",
    )

    list_filter = ("legislative_session__jurisdiction__name",)



# Generated by Django 3.0.5 on 2020-08-05 14:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("data", "0013_remove_person_current_state")]

    operations = [
        migrations.RemoveField(model_name="person", name="current_role_division_id")
    ]



# Generated by Django 3.0.5 on 2020-09-14 11:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("data", "0023_remove_personcontactdetail_label"),
    ]

    operations = [
        migrations.RemoveField(model_name="post", name="end_date",),
        migrations.RemoveField(model_name="post", name="start_date",),
    ]



from ..models import State, Chamber, simple_numbered_districts

VA = State(
    name="Virginia",
    abbr="VA",
    capital="Richmond",
    capital_tz="America/New_York",
    fips="51",
    unicameral=False,
    legislature_name="Virginia General Assembly",
    legislature_organization_id="ocd-organization/e56c1fc3-f311-4a51-a693-05c3e922f1b7",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/1251b370-e115-4240-bd6c-68bfe247050e",
    division_id="ocd-division/country:us/state:va",
    jurisdiction_id="ocd-jurisdiction/country:us/state:va/government",
    url="http://virginiageneralassembly.gov/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/0c3daf42-2835-49cf-8f58-b1d771dae687",
        num_seats=100,
        title="Delegate",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:va", "lower", 100
        ),
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/9f1a0a17-d2fb-4d0c-90fb-119004411b83",
        num_seats=40,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:va", "upper", 40
        ),
    ),
)



# Generated by Django 3.0.5 on 2020-09-14 11:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("data", "0022_auto_20200914_1054"),
    ]

    operations = [
        migrations.RemoveField(model_name="personcontactdetail", name="label",),
    ]



from django.contrib import admin
from django.template import defaultfilters
from . import base
from .. import models


@admin.register(models.EventLocation)
class EventLocationAdmin(admin.ModelAdmin):
    pass


class EventLinkInline(base.LinkInline):
    model = models.EventLink


class EventSourceInline(base.LinkInline):
    model = models.EventSource


class EventParticipantInline(base.RelatedEntityInline):
    model = models.EventParticipant
    readonly_fields = ("organization", "person")


@admin.register(models.Event)
class EventAdmin(admin.ModelAdmin):
    readonly_fields = ("jurisdiction", "location")
    fields = (
        "name",
        "jurisdiction",
        "location",
        "description",
        "classification",
        "status",
        ("start_date", "end_date", "all_day"),
    )

    def source_link(self, obj):
        source = obj.sources.filter(url__icontains="meetingdetail").get()
        tmpl = u'<a href="{0}" target="_blank">View source</a>'
        return tmpl.format(source.url)

    source_link.short_description = "View source"
    source_link.allow_tags = True

    list_display = ("jurisdiction", "name", "start_date", "source_link")

    inlines = [EventLinkInline, EventSourceInline, EventParticipantInline]


@admin.register(models.EventMedia)
class EventMediaAdmin(admin.ModelAdmin):
    pass


@admin.register(models.EventDocument)
class EventDocumentAdmin(admin.ModelAdmin):
    readonly_fields = ("event",)
    list_display = ("event", "date", "note")


# @admin.register(models.EventDocumentLink)
# class EventDocumentLinkAdmin(base.MimetypeLinkAdmin):
#    readonly_fields = ('document',)
#    list_display = ('document', 'media_type', 'url',)


@admin.register(models.EventSource)
class EventSourceAdmin(admin.ModelAdmin):
    readonly_fields = ("event",)


@admin.register(models.EventParticipant)
class EventParticipantAdmin(admin.ModelAdmin):
    pass


@admin.register(models.EventAgendaItem)
class EventAgendaItemAdmin(admin.ModelAdmin):
    readonly_fields = ("event",)
    fields = ("event", "description", "classification", "order", "subjects", "notes")

    def get_truncated_description(self, obj):
        return defaultfilters.truncatewords(obj.description, 25)

    get_truncated_description.short_description = "Description"

    def get_truncated_event_name(self, obj):
        return defaultfilters.truncatewords(obj.event.name, 8)

    get_truncated_event_name.short_description = "Event Name"

    list_display = ("get_truncated_event_name", "get_truncated_description")


@admin.register(models.EventRelatedEntity)
class EventRelatedEntityAdmin(admin.ModelAdmin):
    pass


@admin.register(models.EventAgendaMedia)
class EventAgendaMediaAdmin(admin.ModelAdmin):
    pass


@admin.register(models.EventAgendaMediaLink)
class EventAgendaMediaLinkAdmin(admin.ModelAdmin):
    pass



"""
    Transformers for IMPORT_TRANSFORMERS
"""
import re


_bill_id_re = re.compile(r"([A-Z]*)\s*0*([-\d]+)")
_mi_bill_id_re = re.compile(r"(SJR|HJR)\s*([A-Z]+)")


def fix_bill_id(bill_id):
    bill_id = bill_id.upper()
    # special case for MI Joint Resolutions
    if _mi_bill_id_re.match(bill_id):
        return _mi_bill_id_re.sub(r"\1 \2", bill_id, 1).strip()
    return _bill_id_re.sub(r"\1 \2", bill_id, 1).strip()



from ..models import State, Chamber, simple_numbered_districts

MT = State(
    name="Montana",
    abbr="MT",
    capital="Helena",
    capital_tz="America/Denver",
    fips="30",
    unicameral=False,
    legislature_name="Montana Legislature",
    legislature_organization_id="ocd-organization/803c558f-dec2-4f43-9c35-e73a6e849408",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/2838c5e6-e793-518d-a722-6140d5f8d0aa",
    division_id="ocd-division/country:us/state:mt",
    jurisdiction_id="ocd-jurisdiction/country:us/state:mt/government",
    url="http://leg.mt.gov/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/7508efaf-3ca6-436d-8858-45b2f73a78d9",
        num_seats=100,
        title="Representative",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:mt", "lower", 100
        ),
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/1cf80fcd-e90e-4d0b-87d6-304fad2262f9",
        num_seats=50,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:mt", "upper", 50
        ),
    ),
)



from .base import BaseImporter
from ..exceptions import InternalError
from ..data.models import (
    Bill,
    RelatedBill,
    BillAbstract,
    BillTitle,
    BillIdentifier,
    BillAction,
    BillActionRelatedEntity,
    BillSponsorship,
    BillSource,
    BillDocument,
    BillVersion,
    BillDocumentLink,
    BillVersionLink,
)
from .computed_fields import update_bill_fields
from .people import PersonImporter
from .organizations import OrganizationImporter


class BillImporter(BaseImporter):
    _type = "bill"
    model_class = Bill
    related_models = {
        "abstracts": (BillAbstract, "bill_id", {}),
        "other_titles": (BillTitle, "bill_id", {}),
        "other_identifiers": (BillIdentifier, "bill_id", {}),
        "actions": (
            BillAction,
            "bill_id",
            {"related_entities": (BillActionRelatedEntity, "action_id", {})},
        ),
        "related_bills": (RelatedBill, "bill_id", {}),
        "sponsorships": (BillSponsorship, "bill_id", {}),
        "sources": (BillSource, "bill_id", {}),
        "documents": (
            BillDocument,
            "bill_id",
            {"links": (BillDocumentLink, "document_id", {})},
        ),
        "versions": (
            BillVersion,
            "bill_id",
            {"links": (BillVersionLink, "version_id", {})},
        ),
    }
    preserve_order = {"actions"}

    def __init__(self, jurisdiction_id):
        super(BillImporter, self).__init__(jurisdiction_id)
        self.org_importer = OrganizationImporter(jurisdiction_id)
        self.person_importer = PersonImporter(jurisdiction_id)

    def get_object(self, bill):
        spec = {
            "legislative_session_id": bill["legislative_session_id"],
            "identifier": bill["identifier"],
        }
        # bills should be unique per session + identifier, no need to use organization
        #   (note: this used to be different, but allowed duplicates)

        return self.model_class.objects.prefetch_related(
            "actions__related_entities", "versions__links", "documents__links"
        ).get(**spec)

    def limit_spec(self, spec):
        spec["legislative_session__jurisdiction_id"] = self.jurisdiction_id
        return spec

    def prepare_for_db(self, data):
        data["legislative_session_id"] = self.get_session_id(
            data.pop("legislative_session")
        )

        if data["from_organization"]:
            data["from_organization_id"] = self.org_importer.resolve_json_id(
                data.pop("from_organization")
            )

        for action in data["actions"]:
            action["organization_id"] = self.org_importer.resolve_json_id(
                action["organization_id"]
            )
            for entity in action["related_entities"]:
                if "organization_id" in entity:
                    entity["organization_id"] = self.org_importer.resolve_json_id(
                        entity["organization_id"]
                    )
                elif "person_id" in entity:
                    entity["person_id"] = self.person_importer.resolve_json_id(
                        entity["person_id"]
                    )

        for sponsor in data["sponsorships"]:
            if "person_id" in sponsor:
                sponsor["person_id"] = self.person_importer.resolve_json_id(
                    sponsor["person_id"], allow_no_match=True
                )

            if "organization_id" in sponsor:
                sponsor["organization_id"] = self.org_importer.resolve_json_id(
                    sponsor["organization_id"], allow_no_match=True
                )

        return data

    def postimport(self):
        # go through all RelatedBill objs that are attached to a bill in this jurisdiction and
        # are currently unresolved
        for rb in RelatedBill.objects.filter(
            bill__legislative_session__jurisdiction_id=self.jurisdiction_id,
            related_bill=None,
        ):
            candidates = list(
                Bill.objects.filter(
                    legislative_session__identifier=rb.legislative_session,
                    legislative_session__jurisdiction_id=self.jurisdiction_id,
                    identifier=rb.identifier,
                )
            )
            if len(candidates) == 1:
                rb.related_bill = candidates[0]
                rb.save()
            elif len(candidates) > 1:  # pragma: no cover
                # if we ever see this, we need to add additional fields on the relation
                raise InternalError(
                    "multiple related_bill candidates found for {}".format(rb)
                )

    def update_computed_fields(self, obj):
        update_bill_fields(obj, save=False)



from django.db.models import Q
from .base import BaseImporter
from ..data.models import Person


class PersonImporter(BaseImporter):
    _type = "person"
    model_class = Person

    def limit_spec(self, spec):
        """
        Whenever we do a Pseudo ID lookup from the database, we need to limit
        based on the memberships -> organization -> jurisdiction, so we scope
        the resolution.
        """
        if list(spec.keys()) == ["name"]:
            # if we're just resolving on name, include other names and family name
            name = spec["name"]
            return (Q(name=name) | Q(other_names__name=name) | Q(family_name=name)) & Q(
                memberships__organization__jurisdiction_id=self.jurisdiction_id
            )
        spec["memberships__organization__jurisdiction_id"] = self.jurisdiction_id
        return spec



import pytest
from datetime import datetime
from openstates.data.models import (
    Jurisdiction,
    Division,
    Membership,
    Organization,
    Person,
    Post,
    LegislativeSession,
    Event,
    EventLocation,
    VoteEvent,
    Bill,
)


@pytest.fixture
def division():
    div = Division.objects.create(
        id="ocd-division/country:us/state:mo", name="Missouri"
    )
    return div


@pytest.fixture
def jurisdiction(division):
    juris = Jurisdiction.objects.create(
        id="ocd-jurisdiction/country:us/state:mo/government",
        name="Missouri State Senate",
        url="http://www.senate.mo.gov",
        division=division,
    )
    return juris


@pytest.fixture
def legislative_session(jurisdiction):
    l_s = LegislativeSession.objects.create(
        jurisdiction=jurisdiction,
        identifier=2017,
        name="2017 Session",
        start_date="2017-01-04",
        end_date="2017-05-25",
    )
    return l_s


@pytest.fixture
def organization():
    org = Organization.objects.create(name="Missouri State Senate")
    return org


@pytest.fixture
def bill(legislative_session):
    b = Bill.objects.create(
        legislative_session=legislative_session,
        identifier="HR 3590",
        title="The Patient Protection and Affordable Care Act",
    )
    return b


@pytest.fixture
def vote_event(legislative_session, organization):
    v_e = VoteEvent.objects.create(
        motion_text="That the House do now proceed to the Orders of the Day.",
        start_date="2017-02-16",
        result="pass",
        organization=organization,
        legislative_session=legislative_session,
    )
    return v_e


@pytest.fixture
def event_location(jurisdiction):
    loc = EventLocation.objects.create(
        name="State Legislative Building", jurisdiction=jurisdiction,
    )
    return loc


@pytest.fixture
def event(jurisdiction, event_location):
    e = Event.objects.create(
        name="Meeting of the Committee on Energy",
        jurisdiction=jurisdiction,
        description="To discuss the pros/cons of wind farming.",
        classification="committee-meeting",
        start_date=datetime.utcnow().isoformat().split(".")[0],
        status="passed",
        location=event_location,
    )
    return e


@pytest.fixture
def party():
    p = Organization.objects.create(name="Republican", classification="party")
    return p


@pytest.fixture
def person():
    p = Person.objects.create(name="Arnold Schwarzenegger")
    return p


@pytest.fixture
def senator(jurisdiction, party):
    div_id = "ocd-division/country:us/state:mo/sldu:1"
    person = Person.objects.create(
        name="Willy Worm", current_role={"org_classification": "upper", "district": 1}
    )
    senate = Organization.objects.create(
        classification="upper", name="Senate", jurisdiction=jurisdiction
    )
    division = Division.objects.create(id=div_id, name="MO 1")
    post = Post.objects.create(
        organization=senate, division=division, role="Senator", label="1"
    )
    person.memberships.create(organization=senate, post=post)
    person.memberships.create(organization=party)
    return person


@pytest.fixture
def post(organization):
    p = Post.objects.create(organization=organization, label="Governor")
    return p


@pytest.fixture
def membership(organization, post, person):
    m = Membership.objects.create(organization=organization, post=post, person=person)
    return m



import pytest
from .. import lookup, lookup_district_with_ancestors
from ..data import NC, NE


def test_lookup():
    assert lookup(abbr="nc") == NC
    assert lookup(abbr="NC") == NC
    assert (
        lookup(jurisdiction_id="ocd-jurisdiction/country:us/state:nc/government") == NC
    )


def test_chambers():
    assert len(NC.chambers) == 2
    assert len(NE.chambers) == 1


def test_lookup_district_by_division_id():
    # lookup works
    assert NC.lookup_district("ocd-division/country:us/state:nc/sldl:1") is not None
    assert NC.lookup_district("ocd-division/country:us/state:nc/sldl:1").name == "1"
    # correct chambers
    assert (
        NC.lookup_district("ocd-division/country:us/state:nc/sldl:1").chamber_type
        == "lower"
    )
    assert (
        NC.lookup_district("ocd-division/country:us/state:nc/sldu:1").chamber_type
        == "upper"
    )
    # unicameral
    assert (
        NE.lookup_district("ocd-division/country:us/state:ne/sldu:1").chamber_type
        == "legislature"
    )


def test_lookup_district_by_name():
    assert (
        NC.lookup_district(chamber="upper", name="1").division_id
        == "ocd-division/country:us/state:nc/sldu:1"
    )
    assert (
        NC.lookup_district(chamber="lower", name="1").division_id
        == "ocd-division/country:us/state:nc/sldl:1"
    )
    assert (
        NE.lookup_district(name="1").division_id
        == "ocd-division/country:us/state:ne/sldu:1"
    )


def test_lookup_missing_district():
    assert NE.lookup_district("ocd-division/country:us/state:ne/sldl:1") is None
    assert NE.lookup_district(name="999") is None
    assert NC.lookup_district(name="999") is None
    assert NC.lookup_district(name="999", chamber="lower") is None


def test_lookup_district_with_ancestors():
    state, chamber, district = lookup_district_with_ancestors(
        division_id="ocd-division/country:us/state:nc/sldl:1"
    )
    assert state.name == "North Carolina"
    assert chamber.chamber_type == "lower"
    assert district.name == "1"

    state, chamber, district = lookup_district_with_ancestors(
        division_id="ocd-division/country:us/state:ne/sldu:1"
    )
    assert state.name == "Nebraska"
    assert chamber.chamber_type == "unicameral"
    assert district.name == "1"


def test_lookup_district_with_ancestors_invalid():
    # bad id
    with pytest.raises(ValueError):
        lookup_district_with_ancestors(division_id="invalid id")
    # bad state
    with pytest.raises(ValueError):
        lookup_district_with_ancestors(
            division_id="ocd-division/country:us/state:xy/sldl:1"
        )
    # bad district
    with pytest.raises(ValueError):
        lookup_district_with_ancestors(
            division_id="ocd-division/country:us/state:nc/sldl:999"
        )


def test_lookup_district_with_ancestors_dc_pr():
    state, chamber, district = lookup_district_with_ancestors(
        division_id="ocd-division/country:us/district:dc"
    )
    assert state.name == "District of Columbia"
    assert chamber.chamber_type == "unicameral"
    assert district.name == "At-Large"
    state, chamber, district = lookup_district_with_ancestors(
        division_id="ocd-division/country:us/territory:pr"
    )
    assert state.name == "Puerto Rico"
    assert chamber.chamber_type == "upper"
    assert district.name == "At-Large"



import os
import json
import pytz
import datetime
import subprocess


def utcnow():
    return datetime.datetime.now(datetime.timezone.utc)


def _make_pseudo_id(**kwargs):
    """ pseudo ids are just JSON """
    # ensure keys are sorted so that these are deterministic
    return "~" + json.dumps(kwargs, sort_keys=True)


def get_pseudo_id(pid):
    if pid[0] != "~":
        raise ValueError("pseudo id doesn't start with ~")
    return json.loads(pid[1:])


def makedirs(dname):
    if not os.path.isdir(dname):
        os.makedirs(dname)


class JSONEncoderPlus(json.JSONEncoder):
    """
    JSONEncoder that encodes datetime objects as Unix timestamps.
    """

    def default(self, obj, **kwargs):
        if isinstance(obj, datetime.datetime):
            if obj.tzinfo is None:
                raise TypeError("date '%s' is not fully timezone qualified." % (obj))
            obj = obj.astimezone(pytz.UTC)
            return "{}".format(obj.isoformat())
        elif isinstance(obj, datetime.date):
            return "{}".format(obj.isoformat())
        return super(JSONEncoderPlus, self).default(obj, **kwargs)


def convert_pdf(filename, type="xml"):
    commands = {
        "text": ["pdftotext", "-layout", filename, "-"],
        "text-nolayout": ["pdftotext", filename, "-"],
        "xml": ["pdftohtml", "-xml", "-stdout", filename],
        "html": ["pdftohtml", "-stdout", filename],
    }
    try:
        pipe = subprocess.Popen(
            commands[type], stdout=subprocess.PIPE, close_fds=True
        ).stdout
    except OSError as e:
        raise EnvironmentError(
            "error running %s, missing executable? [%s]" % " ".join(commands[type]), e
        )
    data = pipe.read()
    pipe.close()
    return data


def format_datetime(dt, timezone):
    return pytz.timezone(timezone).localize(dt).replace(microsecond=0).isoformat()



from django.db import models
from ..common import JURISDICTION_CLASSIFICATION_CHOICES, SESSION_CLASSIFICATION_CHOICES
from .base import OCDBase, OCDIDField, RelatedBase
from .division import Division


class Jurisdiction(OCDBase):
    """
    A Jurisdiction represents a logical unit of governance.

    Examples would include: the United States Federal Government, the Government
    of the District of Columbia, the Lexington-Fayette Urban County Government,
    or the Wake County Public School System.
    """

    id = OCDIDField(ocd_type="jurisdiction")
    name = models.CharField(
        max_length=300,
        help_text="The common name of the Jurisdiction, such as 'Wyoming.'",
    )
    url = models.URLField(
        max_length=2000, help_text="The primary website of the Jurisdiction."
    )
    classification = models.CharField(
        max_length=50,
        choices=JURISDICTION_CLASSIFICATION_CHOICES,
        default="state",
        db_index=True,
        help_text="The type of Jurisdiction being defined.",
    )
    division = models.ForeignKey(
        Division,
        related_name="jurisdictions",
        db_index=True,
        null=True,
        help_text="A link to a Division related to this Jurisdiction.",
        # don't allow deletion of a division that a Jurisdiction depends upon
        on_delete=models.PROTECT,
    )

    class Meta:
        db_table = "opencivicdata_jurisdiction"

    def __str__(self):
        return self.name


class LegislativeSession(RelatedBase):
    jurisdiction = models.ForeignKey(
        Jurisdiction,
        related_name="legislative_sessions",
        # should be hard to delete Jurisdiction
        on_delete=models.PROTECT,
    )
    identifier = models.CharField(max_length=100)
    name = models.CharField(max_length=300)
    classification = models.CharField(
        max_length=100, choices=SESSION_CLASSIFICATION_CHOICES, blank=True
    )
    start_date = models.CharField(max_length=10)  # YYYY[-MM[-DD]]
    end_date = models.CharField(max_length=10)  # YYYY[-MM[-DD]]

    def __str__(self):
        return "{} {}".format(self.jurisdiction, self.name)

    class Meta:
        db_table = "opencivicdata_legislativesession"



import attr
import typing


@attr.s
class District:
    name = attr.ib()
    chamber_type = attr.ib()
    division_id = attr.ib()
    num_seats = attr.ib(default=1)


@attr.s(auto_attribs=True)
class Chamber:
    chamber_type: str
    name: str
    title: str
    num_seats: int
    organization_id: str
    districts: typing.List[District]

    def lookup_district(self, division_id=None, *, name=None):
        for d in self.districts:
            if division_id and d.division_id == division_id:
                return d
            if name and d.name == name:
                return d


@attr.s(auto_attribs=True)
class State:
    name: str
    abbr: str
    capital: str
    capital_tz: str
    fips: str
    unicameral: bool
    legislature_name: str
    legislature_organization_id: str
    executive_name: str
    executive_organization_id: str
    division_id: str
    jurisdiction_id: str
    url: str
    lower: Chamber = None
    upper: Chamber = None
    legislature: Chamber = None

    @property
    def chambers(self):
        if self.unicameral:
            return [self.legislature]
        else:
            return [self.lower, self.upper]

    @property
    def legacy_districts(self):
        from .data.legacy_districts import legacy_districts

        return legacy_districts.get(self.abbr.lower(), [])

    def lookup_district(self, division_id=None, *, name=None, chamber=None):
        if self.unicameral:
            return self.legislature.lookup_district(division_id=division_id, name=name)
        elif division_id and "sldl" in division_id or chamber == "lower":
            return self.lower.lookup_district(division_id=division_id, name=name)
        elif division_id and "sldu" in division_id or chamber == "upper":
            return self.upper.lookup_district(division_id=division_id, name=name)


def simple_numbered_districts(parent_id, chamber_type, total, *, num_seats=1):
    prefix = "sldl" if chamber_type == "lower" else "sldu"
    return [
        District(str(n), chamber_type, f"{parent_id}/{prefix}:{n}", num_seats)
        for n in range(1, total + 1)
    ]



from ..models import State, Chamber, simple_numbered_districts

NC = State(
    name="North Carolina",
    abbr="NC",
    capital="Raleigh",
    capital_tz="America/New_York",
    fips="37",
    unicameral=False,
    legislature_name="North Carolina General Assembly",
    legislature_organization_id="ocd-organization/32aab083-d7a0-44e0-9b95-a7790c542605",
    executive_name="Office of the Governor",
    executive_organization_id="ocd-organization/097fcec3-4290-42eb-927f-0a786b873093",
    division_id="ocd-division/country:us/state:nc",
    jurisdiction_id="ocd-jurisdiction/country:us/state:nc/government",
    url="http://www.ncleg.net/",
    lower=Chamber(
        chamber_type="lower",
        name="House",
        organization_id="ocd-organization/4861d484-9f30-411a-baaf-c2c12d4d174f",
        num_seats=120,
        title="Representative",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:nc", "lower", 120
        ),
    ),
    upper=Chamber(
        chamber_type="upper",
        name="Senate",
        organization_id="ocd-organization/6347dffa-4778-4dc0-97d7-fd4db3ff5328",
        num_seats=50,
        title="Senator",
        districts=simple_numbered_districts(
            "ocd-division/country:us/state:nc", "upper", 50
        ),
    ),
)



# Generated by Django 3.0.5 on 2020-08-11 10:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("data", "0015_auto_20200811_1009"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="voteevent", options={"ordering": ["start_date", "order"]},
        ),
        migrations.RemoveField(model_name="voteevent", name="end_date",),
        migrations.AddField(
            model_name="voteevent",
            name="order",
            field=models.PositiveIntegerField(default=0),
        ),
    ]
